package com.engine.jucailinkq.attendance.attendanceanalysis.cmd.item;

import com.engine.common.biz.AbstractCommonCommand;
import com.engine.common.entity.BizLogContext;
import com.engine.core.interceptor.CommandContext;
import com.engine.jucailinkq.common.util.DateUtil;
import com.engine.jucailinkq.common.util.ExtensionClassHolder;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.google.common.collect.Sets;
import lombok.extern.slf4j.Slf4j;
import weaver.general.Util;
import weaver.hrm.company.DepartmentComInfo;

import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

@Slf4j
public class GerPersonAttendanceItemsCmd extends AbstractCommonCommand<Map<String,Object>> {

    public GerPersonAttendanceItemsCmd(Map<String, Object> params){
        this.params=params;
    }
    @Override
    public BizLogContext getLogContext() {
        return null;
    }

    @Override
    public Map<String, Object> execute(CommandContext commandContext) {
        String userId = Util.null2String(params.get("userId"));
        String departmentId = Util.null2String(params.get("departmentId"));
        String subCompanyId = Util.null2String(params.get("subCompanyId"));
        String anysisyDate = Util.null2String(params.get("anysisyDate"));
        String seclevel = Util.null2String(params.get("seclevel"));
        Map<String, List<Map<String, String>>> personData = (Map<String, List<Map<String, String>>>)params.get("personData");
        Map<String, List<Map<String, String>>> personOrganzationMap = (Map<String, List<Map<String, String>>>)params.get("personOrganzationMap");
        Map<String, List<Map<String, String>>> departMentMap = (Map<String, List<Map<String, String>>>)params.get("departMentMap");
        Map<String, List<Map<String, String>>> subCompanyMap = (Map<String, List<Map<String, String>>>)params.get("subCompanyMap");

        Set<String> attendanceItems = Sets.newHashSet();
        //人力资源
        if (personData.get(userId) != null){
            for (Map<String, String> personMap :personData.get(userId)){
                attendanceItems.add(personMap.get("dataid"));
            }
        }

        //人员分组
        Map<String,List<Map<String,Object>>> personBelongGroup = ExtensionClassHolder.getPersonBelongGroupThreadLocal();
        List<Map<String,Object>> personnelGroups = personBelongGroup.get(userId) == null? Lists.newArrayList():personBelongGroup.get(userId);
        personnelGroups = personnelGroups.stream().filter(e->(Util.null2String(e.get("bdate")).equals("") || DateUtil.getTime(e.get("bdate").toString()).compareTo(DateUtil.getTime(anysisyDate))<=0) &&
                (Util.null2String(e.get("edate")).equals("") || DateUtil.getTime(e.get("edate").toString()).compareTo(DateUtil.getTime(anysisyDate))>=0)).collect(Collectors.toList());

        Set<String> personnelGroupIds = personnelGroups.stream().map(e->e.get("mainid").toString()).collect(Collectors.toSet());
        for (Map.Entry<String, List<Map<String, String>>> entry: personOrganzationMap.entrySet()){
            String personnelGroupId = entry.getKey();
            if (personnelGroupIds.contains(personnelGroupId)){
                for (Map<String, String> personGroupMap :entry.getValue()){
                    attendanceItems.add(personGroupMap.get("dataid"));
                }
            }
        }
        //部门
        try {
            Map<String,String> petDepart = ExtensionClassHolder.getPetDepartMentThreadLocal();
            String pdeptids = "";
            if (petDepart != null){
                pdeptids = petDepart.get(departmentId);
            }
            if (pdeptids == null || pdeptids.equals("")){
                pdeptids = new DepartmentComInfo().getAllParentDepartId(Util.null2String(params.get("departmentId")), pdeptids);
                pdeptids = departmentId + pdeptids;
            }
            for (Map.Entry<String, List<Map<String, String>>> entry: departMentMap.entrySet()){
                String id = entry.getKey();
                for (String pdeptid : pdeptids.split(",")){
                    if (pdeptid.equals(id)){
                        for (Map<String, String> map :entry.getValue()){
                            attendanceItems.add(map.get("dataid"));
                        }
                    }
                }
            }
        }catch (Exception e){
            log.error("catch error :{}",e);
        }
        //分部
        if (subCompanyMap.get(subCompanyId) != null){
            for (Map<String, String> map :subCompanyMap.get(subCompanyId)){
                attendanceItems.add(map.get("dataid"));
            }
        }
        Map<String,Object> resultMap = Maps.newHashMap();
        resultMap.put("attendanceItems",attendanceItems);
        return resultMap;
    }
}
