package com.engine.jucailinkq.attendance.attendanceanalysis.job;


import com.engine.jucailinkq.attendance.attendanceanalysis.service.UtilService;
import com.engine.jucailinkq.attendance.attendanceanalysis.service.impl.UtilServiceImpl;
import com.engine.jucailinkq.attendance.attendanceanalysis.wrapper.AttendanceAnalysisWrapper;
import com.engine.jucailinkq.attendance.attendanceplan.service.AttendancePlanService;
import com.engine.jucailinkq.attendance.attendanceplan.service.impl.AttendancePlanServiceImpl;
import com.engine.jucailinkq.attendance.workflow.service.AllowanceService;
import com.engine.jucailinkq.attendance.workflow.service.impl.AllowanceServiceImpl;
import com.engine.common.util.*;
import com.engine.jucailinkq.common.cmd.GetPersonBySuitOrganzationCmd;
import com.engine.jucailinkq.common.exception.AttendanceRunTimeException;
import com.engine.jucailinkq.common.service.CommonService;
import com.engine.jucailinkq.common.service.impl.CommonServiceImpl;
import com.engine.jucailinkq.common.util.*;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.google.common.collect.Sets;
import com.icbc.api.internal.apache.http.impl.cookie.S;
import lombok.extern.slf4j.Slf4j;
import weaver.general.Util;
import weaver.interfaces.schedule.BaseCronJob;

import java.time.ZoneOffset;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

@Slf4j
public class AttendanceAnalysisJob extends BaseCronJob {
    private AttendancePlanService basicsetService = ServiceUtil.getService(AttendancePlanServiceImpl.class);

    private AttendanceAnalysisWrapper attendanceAnalysisService = ServiceUtil.getService(AttendanceAnalysisWrapper.class);

    private AllowanceService allowanceService = ServiceUtil.getService(AllowanceServiceImpl.class);

    private UtilService utilService = ServiceUtil.getService(UtilServiceImpl.class);
    private CommonService commonService = ServiceUtil.getService(CommonServiceImpl.class);

    private String executeDate;
    @Override
    public void execute() {
        log.info("********AttendanceAnalysisJob  start********");

        String useDate = DateUtil.getCurrentDate();
//        if (executeDate != null && !"".equals(executeDate)){
//            useDate = executeDate;
//        }
        log.info("********AttendanceAnalysisJob  executeDate :[{}]********",useDate);

        log.info("********AttendanceAnalysisJob  prepare start******");
        //初始化扩展类
        CommonUtil.initExtensionClassHolder();
        //入职离职当天是否需要进行考勤分析。0：入职离职当天均分析、1：入职离职当天不分析
        String leaveAndEntry = Util.null2String(ExtensionClassHolder.getGlobalSetMap().get("leaveAndEntry"));


        //查询所有在职人员
        String queryUserSql = "select id,departmentid,subcompanyid1,companystartdate,seclevel from hrmresource where status <> '5' and status <> '4' and status <> '7'";
        List<Map<String,Object>> userList = DbTools.getSqlToList(queryUserSql);


        if ("0".equals(leaveAndEntry) || leaveAndEntry.equals("")){
            String queryDepartEmployeeSql = "select resourceid ,changedate  from HRMSTATUSHISTORY  where type_n = 5 and changedate='"+DateUtil.beforeDay(useDate,1)+"'";
            List<String> departEmployeeList = Lists.newArrayList(DbTools.getSqlToList(queryDepartEmployeeSql).stream().map(e->e.get("resourceid").toString()).collect(Collectors.toSet()));
            List<List<String>> departEmployeePartition = Lists.partition(departEmployeeList,200);

            for (List<String> userIdPartition :departEmployeePartition){
                String sql = "select id,departmentid,subcompanyid1,companystartdate from hrmresource where id in ("+String.join(",",userIdPartition)+")";
                userList.addAll(DbTools.getSqlToList(sql));
            }
        }

        Map<String, Object> getAttendanceparam = Maps.newHashMap();
        getAttendanceparam.put("userList",userList);
        getAttendanceparam.put("executeDate",useDate);

        Map<String, Object> attendanceCycle = utilService.getAttendanceCycle(getAttendanceparam);
        Set<String> allDates = Sets.newHashSet();
        for (Map.Entry<String, Object> entry : attendanceCycle.entrySet()){
            Set<String> sets = (Set<String>)entry.getValue();
            for (String date :sets){
                if (DateUtil.getTime(date).compareTo(DateUtil.getTime(useDate)) < 0){
                    allDates.add(date);
                    allDates.add(DateUtil.beforeDay(date,1));
                    allDates.add(DateUtil.AfterDay(date,1));
                }
            }
        }



        List<String> allDateList = Lists.newArrayList(allDates);
        allDateList = allDateList.stream().sorted(Comparator.comparing(e -> DateUtil.getTime(e).toInstant(ZoneOffset.of("+8")).toEpochMilli())).collect(Collectors.toList());
        allDateList = allDateList.stream().map(e->"'"+e+"'").collect(Collectors.toList());

        log.info("all need excute date : [{}]",allDateList);
        List<List<String>> needAnalysisDateList = Lists.partition(allDateList,200);
        List<Map<String,Object>> dataList = Lists.newArrayList();
        List<Map<String,Object>> attendaceResult = Lists.newArrayList();
        for (List<String> list:needAnalysisDateList){
            String dateStrs = String.join(",",list);
            String sql = "select id,userid,signdate,signtime from hrmschedulesign where isincom=1 and signdate in ("+dateStrs+") order by signdate,signtime";
            List<Map<String,Object>> hrmschedulesignList = DbTools.getSqlToList(sql);
            if (hrmschedulesignList.size() > 0){
                dataList.addAll(hrmschedulesignList);
            }

            //补打卡记录
            sql = "select id,bdkry userid,dkrq signdate,dksj signtime from uf_jcl_kq_bdkjl where dkrq in ("+dateStrs+") and jlzt=1 order by dkrq,dksj";
            List<Map<String,Object>> makeUpList = DbTools.getSqlToList(sql);
            if (makeUpList.size() > 0){
                dataList.addAll(makeUpList);
            }

            sql = "select sjzt,sgsj,bc,fbid,bm,ygid,rq,j1,c1,j2,c2,j3,c3,j4,c4,j5,c5,j6,c6,j7,c7,j8,c8 from uf_jcl_kq_cqjg where rq in ("+dateStrs+")";
            List<Map<String,Object>> resultList = DbTools.getSqlToList(sql);
            if (resultList.size() > 0){
                attendaceResult.addAll(resultList);
            }
        }


        dataList = dataList.stream().sorted(Comparator.comparing(e->DateUtil.getTime(e.get("signdate")+" "+e.get("signtime")).toInstant(ZoneOffset.of("+8")).toEpochMilli())).collect(Collectors.toList());
        Map<String, List<Map<String,Object>>> collect = dataList.stream().collect(Collectors.groupingBy(e -> Util.null2String(e.get("userid"))));

        Map<String,List<Map<String,Object>>> attendaceGroupById = attendaceResult.stream().collect(Collectors.groupingBy(e->e.get("ygid").toString()));

        /**考勤项目*/
        Map<String,Object> params = Maps.newHashMap();
        String modeId = Utils.getFormmodeIdMap().get("uf_jcl_kq_kqfa");
        params.put("modeId",modeId);
        Map<String,Object> attendancePlanMap = commonService.GetAllPersonBySuitOrganzationCmd(params);
        Map<String,Object> attendacnePlanItems = basicsetService.getAttendanceItemsGroupByAttendancePlanDataCmd();

        //通用考勤项目
        List<Map<String,Object>> generalAttendanceItems = (List<Map<String,Object>>)attendacnePlanItems.get("generalAttendanceItems");
        //考勤方案对应所属的考勤项目
        Map<String,List<Map<String,Object>>> attendanceItemGroupByPlan = (Map<String,List<Map<String,Object>>>)attendacnePlanItems.get("attendanceItemGroupByPlan");

        /**综合工时*/
        modeId = Utils.getFormmodeIdMap().get("uf_jcl_kq_zhgs");
        params.put("modeId",modeId);
        Map<String,Object> workingHourResultMap = commonService.getPersonBySuitOrganzation(params);
        //人员所拥有的所有综合工时
        Map<String,List<Map<String,String>>> personComprehensiveWorkingHours = (Map<String,List<Map<String,String>>>)workingHourResultMap.get("personAllData");
        String sql = "select *  from  uf_jcl_kq_zhgs";
        List<Map<String,Object>> resultList = DbTools.getSqlToList(sql);
        //所有综合工时
        Map<String,List<Map<String,Object>>> workingHourGroupBy = resultList.stream().collect(Collectors.groupingBy(e->e.get("id").toString()));

        /**排班结果*/
        List<Map<String,Object>> allScheduleList = Lists.newArrayList();
        /**加班计划*/
        List<Map<String,Object>> allOverPlanList = Lists.newArrayList();
        for (List<String> list:needAnalysisDateList) {
            String dateStrs = String.join(",", list);
            sql = "select a.id as keyid,a.* from uf_pbjg a where bcrq in ("+dateStrs+") order by bcrq";
            List<Map<String,Object>> list1 = DbTools.getSqlToList(sql);
            if (list1.size() > 0){
                allScheduleList.addAll(list1);
            }
            sql = "select a.id,b.jbry,b.ksrq,b.kssj,b.jblx,b.jsrq,b.jssj,b.jbsc,b.gsrq,b.d_jxgz,b.jxsc from uf_jcl_kq_jbjh a left join uf_jcl_kq_jbjh_dt1 b on a.id=b.mainid where  b.gsrq in ("+dateStrs+") and (b.jbcx=0 or b.jbcx is null) and a.jlzt=1";
            List<Map<String,Object>> list2 = DbTools.getSqlToList(sql);
            if (list2.size() > 0){
                allOverPlanList.addAll(list2);
            }
        }
        Map<String,List<Map<String,Object>>> allOverPlanGroup = allOverPlanList.stream().collect(Collectors.groupingBy(e->e.get("jbry").toString()));
        /**班次信息*/
        sql = "select a.xxbdkzdjb,a.sfxx,a.id bcxx,a.edsc,a.zgzsc,a.bcsdxx,a.sfdx,a.sfkt,b.bdlx,a.btgz,a.fgsjd,a.zddxfz,a.dxhs,b.kssj dtkssj,b.jssj dtjssj,b.ksdk,b.jsdk,b.tqdkfzs,b.thdkfzs,b.edxss,b.edfzs,b.edts,b.sfdx dtsfdx,b.zddxfz dtzddxfz,b.dxhs dtdxhs,b.gsrq from uf_jcl_kq_bcxx a left join uf_jcl_kq_bcxx_dt1 b on a.id=b.mainid";
        List<Map<String,Object>> allClassInfoList = DbTools.getSqlToList(sql);
        /**出差信息*/
        String querEvectionsql = "select b.ccr,b.cclx,b.ksrq,b.jsrq,b.kssj,b.jssj,b.ccsc,b.qtcc,b.btcc,b.cxcc from uf_jcl_kq_ccjl a left join uf_jcl_kq_ccjl_dt1 b on a.id = b.mainid where  b.ksrq<="+allDateList.get(allDateList.size()-1)+" and b.jsrq>="+allDateList.get(0)+" and (b.cxcc=0 or b.cxcc is null) and a.jlzt=1";
        Map<String,List<Map<String,Object>>> evectionGroup = DbTools.getSqlToList(querEvectionsql).stream().collect(Collectors.groupingBy(e->e.get("ccr").toString()));
        /**请假信息*/
        String queryAskForsql = "select b.qjry,b.jqlx,b.ksrq,b.kssj,b.jsrq,b.jssj,b.qjsc,b.qtj,b.btj from uf_jcl_kq_qjjl a left join uf_jcl_kq_qjjl_dt1 b on a.id = b.mainid where  b.ksrq<="+allDateList.get(allDateList.size()-1)+" and b.jsrq>="+allDateList.get(0)+" and (b.cxqj=0 or b.cxqj is null) and a.jlzt=1";
        Map<String,List<Map<String,Object>>> askforGroup = DbTools.getSqlToList(queryAskForsql).stream().collect(Collectors.groupingBy(e->e.get("qjry").toString()));
        /**候选班组*/
        String queryHxbz = "select * from uf_jcl_kq_hxbz_dt1";
        List<Map<String,Object>> hxbzList = DbTools.getSqlToList(queryHxbz);


        log.info("********AttendanceAnalysisJob  prepare end******");
        CommonUtil.checkHavePermission();
        for (Map<String,Object> userMap:userList){

            Map<String,Object> analysisParam = Maps.newHashMap();
            String userId = Util.null2String(userMap.get("id"));

            //入职日期
            String companystartDate = Util.null2String(userMap.get("companystartdate"));

            List<String> analysisDateList = Lists.newArrayList((Set<String>)attendanceCycle.get(userId));
            try {
                if (leaveAndEntry.equals("1")){
                    analysisDateList = analysisDateList.stream().filter(e->"".equals(companystartDate) || DateUtil.getTime(e).compareTo(DateUtil.getTime(companystartDate)) >0).sorted(Comparator.comparing(e -> DateUtil.getTime(e).toInstant(ZoneOffset.of("+8")).toEpochMilli())).collect(Collectors.toList());
                }else {
                    analysisDateList = analysisDateList.stream().filter(e->"".equals(companystartDate) || DateUtil.getTime(e).compareTo(DateUtil.getTime(companystartDate)) >=0).sorted(Comparator.comparing(e -> DateUtil.getTime(e).toInstant(ZoneOffset.of("+8")).toEpochMilli())).collect(Collectors.toList());

                }
                if (analysisDateList.size() == 0){
                    throw new AttendanceRunTimeException("分析日期没用值");
                }
                //日期对应的打卡数据
                Map<String,Map<String,Object>> clockInTimeMap = Maps.newHashMap();

                Map<String,Object> paramMap = Maps.newHashMap();
                paramMap.put("startDate",DateUtil.beforeDay(useDate,1));
                paramMap.put("endDate",DateUtil.beforeDay(useDate,1));
                paramMap.put("resourceId",userId);



                analysisParam.put("generalAttendanceItems",generalAttendanceItems);
                analysisParam.put("attendancePlanMap",attendancePlanMap);
                analysisParam.put("attendanceItemGroupByPlan",attendanceItemGroupByPlan);
                analysisParam.put("comprehensiveWorkingHourList",personComprehensiveWorkingHours.get(userId)==null?Lists.newArrayList():personComprehensiveWorkingHours.get(userId));
                analysisParam.put("workingHourGroupBy",workingHourGroupBy);
                analysisParam.put("allScheduleList",allScheduleList);
                analysisParam.put("allClassInfoList",allClassInfoList);
                analysisParam.put("allOverPlanList",allOverPlanGroup.get(userId)==null?Lists.newArrayList():allOverPlanGroup.get(userId));
                List<Map<String,Object>> evectionList = evectionGroup.get(userId) == null?Lists.newArrayList():evectionGroup.get(userId);
                String minAnalysisDate = analysisDateList.get(0);
                String maxAnalysisDate = analysisDateList.get(analysisDateList.size()-1);
                evectionList = evectionList.stream().filter(e->DateUtil.getTime(e.get("ksrq").toString()).compareTo(DateUtil.getTime(maxAnalysisDate)) <=0 &&
                        DateUtil.getTime(e.get("jsrq").toString()).compareTo(DateUtil.getTime(minAnalysisDate)) >=0).collect(Collectors.toList());
                List<Map<String,Object>> askforList = askforGroup.get(userId) == null?Lists.newArrayList():askforGroup.get(userId);
                askforList = askforList.stream().filter(e->DateUtil.getTime(e.get("ksrq").toString()).compareTo(DateUtil.getTime(maxAnalysisDate)) <=0 &&
                        DateUtil.getTime(e.get("jsrq").toString()).compareTo(DateUtil.getTime(minAnalysisDate)) >=0).collect(Collectors.toList());
                analysisParam.put("evectionList",evectionList);
                analysisParam.put("askforList",askforList);
                analysisParam.put("hxbzList",hxbzList);
                analysisParam.put("seclevel",Util.null2String(userMap.get("seclevel")));
                analysisParam.put("departmentId",Util.null2String(userMap.get("departmentid")));
                analysisParam.put("subCompanyId",Util.null2String(userMap.get("subcompanyid1")));


                Map<String, List<Map<String, Object>>> attendaceResultMap = Maps.newHashMap();
                List<Map<String, Object>> attendaceResultByUser =  attendaceGroupById.get(userId);
                if (attendaceResultByUser != null && attendaceResultByUser.size() > 0) {
                    attendaceResultMap = attendaceResultByUser.stream().collect(Collectors.groupingBy(e -> e.get("rq").toString()));
                    for (Map<String, Object> map: attendaceResultByUser){
                        clockInTimeMap.put(map.get("rq").toString(),map);
                    }
                }
                attendanceAnalysisService.attendanceAnalysis(userId,analysisDateList,collect.get(userMap.get("id")),analysisParam,clockInTimeMap,attendaceResultMap);
                
            }catch (Exception e){
                log.error("AttendanceAnalysisJob userId:[{}],analysisDateList: [{}]",userId,analysisDateList);
                log.error("AttendanceAnalysisJob error : [{}]",e);
            }
        }
        Map<String,Object> param = Maps.newHashMap();
        param.put("startDate",DateUtil.beforeDay(useDate,3));
        param.put("endDate",useDate);
        log.info("********AttendanceAnalysisJob analysis end********");
        log.info("********AttendanceAnalysisJob allowance start********");
        allowanceService.addAllowanceRecords(param);
        log.info("********AttendanceAnalysisJob  end********");
    }

}
