package com.engine.jucailinkq.attendance.component.persongroup.service.impl;

import com.engine.jucailinkq.attendance.component.persongroup.cmd.*;
import com.engine.jucailinkq.attendance.component.persongroup.commonutil.PersongroupCommonUtil;
import com.engine.jucailinkq.attendance.component.persongroup.service.SchedulingResultsService;
import com.engine.jucailinkq.attendance.enums.CheckBoxEnum;
import com.engine.jucailinkq.attendance.enums.SchedulingApproachEnum;
import com.engine.jucailinkq.common.util.CommonUtil;
import com.engine.jucailinkq.common.util.DateUtil;
import com.engine.jucailinkq.common.util.DbTools;
import com.engine.jucailinkq.common.util.Utils;
import com.engine.core.impl.Service;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import lombok.extern.slf4j.Slf4j;
import weaver.general.Util;

import java.time.ZoneOffset;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


@Slf4j
public class SchedulingResultsServiceImpl extends Service implements SchedulingResultsService {

    @Override
    public Map<String, Object> queryDataTable(Map<String, Object> params) {
        String recurrence = Util.null2String(params.get("recurrence"));
        Map<String,Object> resultMap = Maps.newHashMap();
        int current = Integer.valueOf(Util.null2String(params.get("current")));
        int pageSize = Integer.valueOf(Util.null2String(params.get("pageSize")));
        String showAll = Util.null2String(params.get("showAll"));
        int startindex = (current-1)*pageSize;
        int endindex = current*pageSize;
//        Map<String,Object> personGroupMap = CommonUtil.getPersonGroup();
//        Map<String,Object> departMentMap = CommonUtil.getDepartMent();
//        Map<String,Object> subCompanyMap = CommonUtil.getSubCompany();
//        Map<String,Object> classesInformationMap = CommonUtil.getClassesInformation();
//        Map<String,Object> candidateTeamMap = CommonUtil.getCandidateTeam();

        if ("1".equals(recurrence)){
            //递归查询方式
            Map<String,Object> result = commandExecutor.execute(new GetDatatableRecurrenceCmd(params));
            List<Map<String, Object>> dataTableList = (List<Map<String, Object>>)result.get("dataTableList");
            if (!CheckBoxEnum.CHECKED.getKey().equals(showAll)){
                dataTableList = Utils.getSchedulingList(dataTableList);
            }
            dataTableList = dataTableList.stream().sorted(Comparator.comparing(e->DateUtil.getTime(e.get("bcrq").toString()).toInstant(ZoneOffset.of("+8")).toEpochMilli())).collect(Collectors.toList());
            int total = dataTableList.size();
            resultMap.put("total",total);
            if (pageSize != 9999){
                if (endindex > total){
                    endindex = total;
                }
                startindex = endindex-pageSize;
                if (startindex < 0){
                    startindex = 0;
                }
                dataTableList = dataTableList.subList(startindex,endindex);
            }
            resultMap.put("data",dataTableList);

        }else {
            //非递归查询方式
            Map<String,Object> result = commandExecutor.execute(new GetDatatableCountCmd(params));
            int total = Integer.valueOf(result.get("total").toString());
            params.put("total",total);
            resultMap = commandExecutor.execute(new GetDatatableCmd(params));
            resultMap.put("total",total);
        }

        return resultMap;
    }


    @Override
    public Map<String, Object> queryDataTableActualUse(Map<String, Object> params) {
        Map<String, Object> paramMap = queryDataTable(params);
        Map<String, Object> resultMap = Maps.newHashMap();
        List<Map<String, Object>> dataTableList = (List<Map<String, Object>>)paramMap.get("data");
        List<Map<String, Object>> resultList = Lists.newArrayList();
        resultMap.put("data",Utils.getSchedulingList(dataTableList));
        return resultMap;
    }

    @Override
    public Map<String, Object> saveScheduleResult(Map<String, Object> params) {
        String billid = Util.null2String(params.get("billid"));
        String sql = "select bc,rqlx,rq,ygid from uf_jcl_kq_cqjg where id=?";
        List<Map<String,Object>> dataList = DbTools.getSqlToList(sql,billid);
        Map<String,Object> dataMap = Maps.newHashMap();
        dataMap.put("pbtj",SchedulingApproachEnum.UPDATE.getKey());
        dataMap.put("dxlx","0");
        dataMap.put("pbdxry",dataList.get(0).get("ygid"));
        dataMap.put("bcrq",dataList.get(0).get("rq"));
        dataMap.put("rqlx",dataList.get(0).get("rqlx"));
        dataMap.put("sfxx", CommonUtil.ifrestByClass(dataList.get(0).get("bc").toString()));
        dataMap.put("sfdkpp", CheckBoxEnum.CHECKED.getKey());
        dataMap.put("bcxx",dataList.get(0).get("bc"));
        dataMap.put("pbrq",DateUtil.getCurrentDate());
        dataMap.put("pbsj",DateUtil.getCurrentTime("yyyy-MM-dd HH:mm").split(" ")[1]);

        Utils.InsertFormTable("uf_pbjg",dataMap,Utils.getFormmodeIdMap());

        return null;
    }

    @Override
    public Map<String, Object> generateSchedule(Map<String, Object> params) {
        return commandExecutor.execute(new GenerateScheduleCmd(params));
    }

    @Override
    public Map<String, Object> deleteBatchSchedule(Map<String, Object> params) {

        return commandExecutor.execute(new DeleteBatchScheduleCmd(params));
    }

    @Override
    public Map<String, Object> getUseSchedule(Map<String, Object> params) {
        Map<String, Object> result = commandExecutor.execute(new GetScheduleFoAnalysisCmd(params));
        Map<String, Object> resultMap = Maps.newHashMap();
        List<Map<String, Object>> dataTableList = (List<Map<String, Object>>)result.get("dataTableList");
        resultMap.put("data",Utils.getSchedulingList(dataTableList));
        return resultMap;
    }
}
