package com.engine.jucailinkq.attendance.vacation.job.holidaygeneration.tactics;


import com.engine.jucailinkq.attendance.enums.CheckBoxEnum;
import com.engine.jucailinkq.attendance.enums.QuotaChangeMethodEnum;
import com.engine.jucailinkq.attendance.enums.StartingUnitEnum;
import com.engine.jucailinkq.attendance.vacation.util.VocationCommonUtil;
import com.engine.jucailinkq.common.util.DateUtil;
import com.google.common.collect.Maps;
import lombok.extern.slf4j.Slf4j;
import weaver.general.Util;

import javax.swing.text.DateFormatter;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Calendar;
import java.util.List;
import java.util.Map;

/**
 * 循环单次发放全额
 */
@Slf4j
public class SinglePaymentInFullWay implements HolidayGenerationWay {
    @Override
    public boolean support(int personClassfiy) {
        if (personClassfiy == 0) {
            return true;
        }
        return false;
    }

    @Override
    public Map<String,Object> compute(Map<String, Object> data) {
        String userId = Util.null2String(data.get("userId"));
        String releaseDate = Util.null2String(data.get("releaseDate"));
        String cover = Util.null2String(data.get("cover"));
        String isOneTime = Util.null2String(data.get("isOneTime"));
        if ("".equals(releaseDate)) {
            releaseDate = DateUtil.getCurrentTime();
        }
        List<Map<String, Object>> vocationList = (List<Map<String, Object>>) data.get("vocationList");
        //起算日期
        String qsrq = Util.null2String(vocationList.get(0).get("qsrqValue"));
        //截止日期
        String jzrqzd = Util.null2String(data.get("leaveDate"));
        //额度有效期间
        String edyxqsz = Util.null2String(vocationList.get(0).get("edyxqsz"));
        //额度有效期间单位
        String edyxq = Util.null2String(vocationList.get(0).get("edyxq"));
        //累计承认工龄月份数栏位值
        String ljcrglyfslwz = Util.null2String(vocationList.get(0).get("ljcrglyfslwzValue")).equals("")?"0":Util.null2String(vocationList.get(0).get("ljcrglyfslwzValue"));
        //依据字段
        String yjzd = Util.null2String(vocationList.get(0).get("yjzdValue"));
        //调整起算日期策略
        String dzqsrq = Util.null2String(vocationList.get(0).get("dzqsrq"));
        //分割日期
        String fgrq = Util.null2String(vocationList.get(0).get("fgrq"));
        //入职当期额度按比例折算
        String rzdqedablzs = Util.null2String(vocationList.get(0).get("rzdqedablzs"));
        //首次获得额度按比例折算
        String schdedablzs = Util.null2String(vocationList.get(0).get("schdedablzs"));
        //额度变更时额度处理方式
        String edbgsedclfs = Util.null2String(vocationList.get(0).get("edbgsedclfs"));
        //额度单位
        String eddw = Util.null2String(vocationList.get(0).get("edbgsedclfs"));
        //尾数处理
        String wscl = Util.null2String(vocationList.get(0).get("wscl"));
        //首次获得额度控制生效日期
        String schdedkzsxrq = Util.null2String(vocationList.get(0).get("schdedkzsxrq"));
        //间隔时长算法
        String jgscsf = Util.null2String(vocationList.get(0).get("jgsc"));
        //累计时长单位：0：年；1：月
        String ljgldw = Util.null2String(vocationList.get(0).get("lsgldw"));

        if ("".equals(qsrq)){
            return Maps.newHashMap();
        }
        if (!"1".equals(isOneTime)){
            qsrq = VocationCommonUtil.getAfterAdjustTime(qsrq, dzqsrq, fgrq);
            for (Map<String, Object> vocationMap : vocationList) {
                vocationMap.put("qsrqValue", qsrq);
            }
        }
        if (DateUtil.getTime(releaseDate).compareTo(DateUtil.getTime(qsrq)) < 0){
            //releaseDate=qsrq;
            return Maps.newHashMap();
        }

        //相差天数
        int betweenDays = DateUtil.getBetWeenDays(qsrq, releaseDate.split(" ")[0]);
        //相差月份
        int betweenMonth = DateUtil.getBetWeenMonthsNeedDayOfMonth(qsrq, releaseDate.split(" ")[0]);
        //总工龄天数
        int allWorkDays = Double.valueOf(ljcrglyfslwz).intValue() * 30 + betweenDays;
        //总月数
        int allMonths = betweenMonth + Double.valueOf(ljcrglyfslwz).intValue();


        Map<String, Object> insertHoliDayParam = VocationCommonUtil.getInsertHoliDayParam(vocationList.get(0), userId, qsrq, releaseDate);

        List<Map<String,Object>> dataList = VocationCommonUtil.ifexist("uf_jcl_kq_jqye", insertHoliDayParam);
       // List<Map<String,Object>> dataList = new ArrayList<>();

        //已存在数据且不覆盖
        if (dataList.size()>0 && !"1".equals(cover)) {
            return insertHoliDayParam;
        }

        String sxrq = insertHoliDayParam.get("sxrq").toString();
        //实际休息时长
        double restTime = 0;
        //工龄小于1年的新员工,且额度有效期单位为自然年
        if (allMonths < 12 && edyxq.equals(StartingUnitEnum.NATURAL_YEAR.getKey())) {

            if (QuotaChangeMethodEnum.CONVERT.getKey().equals(edbgsedclfs) || CheckBoxEnum.CHECKED.getKey().equals(schdedablzs)) {

                restTime = getRestTime(insertHoliDayParam,vocationList,QuotaChangeMethodEnum.CONVERT,userId);
            } else {
                restTime = getRestTime(insertHoliDayParam,vocationList,QuotaChangeMethodEnum.NO_CONVERT,userId);
            }


            if (CheckBoxEnum.CHECKED.getKey().equals(schdedkzsxrq)) {
                int seniority = "".equals(ljcrglyfslwz)?0:Double.valueOf(ljcrglyfslwz).intValue();
                String divideTime = DateUtil.beforeMonth(qsrq,seniority);
                divideTime = DateUtil.nextYear(divideTime,1, DateUtil.yyyyMMdd);
                if (DateUtil.getTime(divideTime).compareTo(DateUtil.getTime(insertHoliDayParam.get("jzrq").toString())) > 0){
                    return insertHoliDayParam;
                }
                insertHoliDayParam.put("sxrq", divideTime);

            }
            insertHoliDayParam.put("ktsc", restTime);
        } else if (allMonths >= 12 && edyxq.equals(StartingUnitEnum.NATURAL_YEAR.getKey())) {
            //入职不满一年，且入职当期额度按比例折算
            if (CheckBoxEnum.CHECKED.getKey().equals(rzdqedablzs) && betweenMonth < 12
                    && sxrq.split("-")[0].equals(releaseDate.split("-")[0]) && releaseDate.split("-")[0].equals(qsrq.split("-")[0])) {
                //起算年和发生日期在同一年，生成假期为起算年的当年
                restTime = getRestTime(insertHoliDayParam,vocationList,QuotaChangeMethodEnum.CONVERT,userId);
            } else {


                if (QuotaChangeMethodEnum.CONVERT.getKey().equals(edbgsedclfs)) {

                    restTime = getRestTime(insertHoliDayParam,vocationList,QuotaChangeMethodEnum.CONVERT,userId);
                } else if (QuotaChangeMethodEnum.NO_CONVERT.getKey().equals(edbgsedclfs)) {
                    restTime = getRestTime(insertHoliDayParam,vocationList,QuotaChangeMethodEnum.NO_CONVERT,userId);
                }
            }
        } else if (edyxq.equals(StartingUnitEnum.STARTING_YEAR.getKey())) {
            int betweenYear = DateUtil.getBetWeenYears(qsrq,releaseDate.split(" ")[0]);
            //起算年
            allMonths = Double.valueOf(ljcrglyfslwz).intValue();
            int intervalYear = allMonths / 12+betweenYear;
            if (jgscsf.equals("2")){
                intervalYear = allMonths / 12;
            }
            Map<String, Object> vocationMap = VocationCommonUtil.getVocationMap(vocationList, intervalYear, yjzd,userId);
            restTime = vocationMap.size()>0?Double.valueOf(vocationMap.get("edktsc").toString()):0;
        } else if (edyxq.equals(StartingUnitEnum.NATURAL_MONTH.getKey())){
            //自然月
            betweenMonth = DateUtil.getBetWeenMonths(qsrq,sxrq);
            if (DateUtil.getTime(sxrq).compareTo(DateUtil.getTime(qsrq)) <=0){
                betweenMonth = 0;
            }
            allMonths = betweenMonth+Double.valueOf(ljcrglyfslwz).intValue();
            if (jgscsf.equals("2")){
                allMonths = Double.valueOf(ljcrglyfslwz).intValue();
            }


            Map<String, Object> vocationMap = VocationCommonUtil.getVocationMap(vocationList, allMonths, yjzd,userId);
            restTime = vocationMap.size()>0?Double.valueOf(vocationMap.get("edktsc").toString()):0;

        } else if (edyxq.equals(StartingUnitEnum.STARTING_MONTH.getKey())){
            //起算月
            betweenMonth = DateUtil.getBetWeenMonths(qsrq,sxrq);
            allMonths = betweenMonth+Double.valueOf(ljcrglyfslwz).intValue();
            if (jgscsf.equals("2")){
                allMonths = Double.valueOf(ljcrglyfslwz).intValue();
            }

            Map<String, Object> vocationMap = VocationCommonUtil.getVocationMap(vocationList, allMonths, yjzd,userId);
            restTime = vocationMap.size()>0?Double.valueOf(vocationMap.get("edktsc").toString()):0;
        } else if (edyxq.equals(StartingUnitEnum.NATURAL_WEEK.getKey())){
            //自然周
            betweenDays = DateUtil.getBetWeenDays(qsrq,sxrq);
            if (DateUtil.getTime(sxrq).compareTo(DateUtil.getTime(qsrq)) <=0){
                betweenDays = 0;
            }
            allWorkDays = Double.valueOf(ljcrglyfslwz).intValue()*30+betweenDays;
            if (jgscsf.equals("2")){
                allWorkDays = Double.valueOf(ljcrglyfslwz).intValue()*30;
            }
            Map<String, Object> vocationMap = VocationCommonUtil.getVocationMap(vocationList, allWorkDays/7, yjzd,userId);
            restTime = vocationMap.size()>0?Double.valueOf(vocationMap.get("edktsc").toString()):0;

        } else if (edyxq.equals(StartingUnitEnum.STARTING_WEEK.getKey())){
            //起算周
            betweenDays = DateUtil.getBetWeenDays(qsrq,sxrq);
            if (betweenDays <=7){
                betweenDays=0;
            }
            allWorkDays = Double.valueOf(ljcrglyfslwz).intValue()*30+betweenDays;
            if (jgscsf.equals("2")){
                allWorkDays = Double.valueOf(ljcrglyfslwz).intValue()*30;
            }
            Map<String, Object> vocationMap = VocationCommonUtil.getVocationMap(vocationList, allWorkDays/7, yjzd,userId);
            restTime = vocationMap.size()>0?Double.valueOf(vocationMap.get("edktsc").toString()):0;

        } else if (edyxq.equals(StartingUnitEnum.NATURAL_DAY.getKey())){
            //自然天
            betweenDays = DateUtil.getBetWeenDays(qsrq,sxrq);
            allWorkDays = Double.valueOf(ljcrglyfslwz).intValue()*30+betweenDays;
            if (jgscsf.equals("2")){
                allWorkDays = Double.valueOf(ljcrglyfslwz).intValue()*30;
            }
            Map<String, Object> vocationMap = VocationCommonUtil.getVocationMap(vocationList, allWorkDays, yjzd,userId);
            restTime = vocationMap.size()>0?Double.valueOf(vocationMap.get("edktsc").toString()):0;
        }
        insertHoliDayParam.put("ktsc", restTime);

        if (!"1".equals(isOneTime)){
            //离职日期
            //入职日期
            List<Map<String,Object>> userList = (List<Map<String,Object>>)data.get("userList");
            Map<String,Object> userMap = userList.size()>0?userList.get(0):Maps.newHashMap();

            String companystartdate = Util.null2String(userMap.get("companystartdate"));
            if ("companystartdate".equals(Util.null2String(vocationList.get(0).get("qsrq")))){
                companystartdate="";
            }

            VocationCommonUtil.computeRestTimeByjzrqzd(insertHoliDayParam,jzrqzd,companystartdate,wscl);
            VocationCommonUtil.updateHolidayBalance("uf_jcl_kq_jqye", insertHoliDayParam,dataList,edyxq);
        }
        return insertHoliDayParam;
    }


    public double getRestTime(Map<String, Object> insertHoliDayParam,List<Map<String, Object>> vocationList,QuotaChangeMethodEnum changeMethodEnum,String userId){
        String sxrq = insertHoliDayParam.get("sxrq").toString();
        String qsrq = Util.null2String(vocationList.get(0).get("qsrqValue"));
        String nextSxrq = DateUtil.nextYear(sxrq,1,DateUtil.yyyyMMdd);
        //累计承认工龄月份数栏位值
        String ljcrglyfslwz = Util.null2String(vocationList.get(0).get("ljcrglyfslwzValue"));
        //依据字段
        String yjzd = Util.null2String(vocationList.get(0).get("yjzdValue"));
        //尾数处理
        String wscl = Util.null2String(vocationList.get(0).get("wscl"));
        //间隔时长算法
        String jgscsf = Util.null2String(vocationList.get(0).get("jgsc"));
        //累计工龄数或者直取工龄数
        int seniority = 0;
        //累计时长单位：0：年；1：月
        String ljgldw = Util.null2String(vocationList.get(0).get("lsgldw"));
        if(!"2".equals(jgscsf)) {
            //间隔时长不为直取工龄时，进行取整处理 todo:实际上累计工龄为年时 基本上都是有小数的 ，都不应该取整 ，这里的逻辑有待优化
            seniority = "".equals(ljcrglyfslwz) ? 0 : Double.valueOf(ljcrglyfslwz).intValue();
        }
        double restTime=0.0;
        //折算
        if (changeMethodEnum == QuotaChangeMethodEnum.CONVERT){
            int beginYear = 0;
            int endYear = 0;
            String divideTime = "";
            if("2".equals(jgscsf)) {
                //直接取工龄字段
                beginYear = Double.valueOf(ljcrglyfslwz).intValue();
                endYear = beginYear+1;
                DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
                double divdemonths = Double.valueOf(ljcrglyfslwz) - Double.valueOf(ljcrglyfslwz).intValue();
                divideTime = DateUtil.nextMonth(sxrq,(int)Math.ceil(divdemonths*12),dateTimeFormatter);
                //LocalDateTime localDateTime = DateUtil.getTime(sxrq);
                //localDateTime = localDateTime.plusMonths((int)Math.ceil(divdemonths*12));
                //divideTime = localDateTime.format(dateTimeFormatter);
            }else {
                int beginYearMonth = DateUtil.getBetWeenMonths(qsrq, sxrq);
                if (beginYearMonth < 0) {
                    beginYearMonth = 0;
                }
                beginYearMonth = beginYearMonth + seniority;
                int endYearMonth = DateUtil.getBetWeenMonths(qsrq, nextSxrq);
                endYearMonth = endYearMonth + seniority;
                beginYear = beginYearMonth / 12;
                endYear = endYearMonth / 12;
                //残年分割日期
                divideTime = DateUtil.beforeMonth(qsrq,seniority);
                divideTime = sxrq.split("-")[0]+"-"+divideTime.split("-")[1]+"-"+divideTime.split("-")[2];
            }
            Map<String,Object> beginYearmap = VocationCommonUtil.getVocationMap(vocationList, beginYear, yjzd,userId);
            Map<String,Object> endYearmap = VocationCommonUtil.getVocationMap(vocationList, endYear, yjzd,userId);
            double beginEdktsc = Double.valueOf(beginYearmap.get("edktsc") == null?  "0" :Util.null2String(beginYearmap.get("edktsc")));
            double endEdktsc = Double.valueOf(endYearmap.get("edktsc") == null?  "0" :Util.null2String(endYearmap.get("edktsc")));

            int yearDays = DateUtil.getDays(sxrq, Calendar.DAY_OF_YEAR);

            if (beginEdktsc == endEdktsc){
                //当前后额定可休时长一样时，代表没有残年情况
                if (DateUtil.getTime(qsrq).compareTo(DateUtil.getTime(sxrq)) >0){
                    int days = DateUtil.getBetWeenDays(qsrq,nextSxrq);
                    restTime = VocationCommonUtil.computeRestDays(days, yearDays, endEdktsc, wscl);
                }else {
                    restTime = beginEdktsc;
                }
            }else {
                //考虑残年
                if (DateUtil.getTime(qsrq).compareTo(DateUtil.getTime(sxrq)) >0){

                    if (DateUtil.getTime(divideTime).compareTo(DateUtil.getTime(qsrq)) >=0){
                        int beforeDays = DateUtil.getBetWeenDays(qsrq,divideTime);
                        int afterDays = DateUtil.getBetWeenDays(divideTime,nextSxrq);

                        double beforeRestTime = VocationCommonUtil.computeRestDays(beforeDays, yearDays, beginEdktsc);
                        double afterRestTime = VocationCommonUtil.computeRestDays(afterDays, yearDays, endEdktsc);
                        restTime = VocationCommonUtil.handleRestDays(beforeRestTime+afterRestTime,wscl);
                    }else {
                        int afterDays = DateUtil.getBetWeenDays(qsrq,nextSxrq);
                        restTime = VocationCommonUtil.computeRestDays(afterDays, yearDays, endEdktsc, wscl);
                    }

                }else {
                    int beforeDays = DateUtil.getBetWeenDays(sxrq,divideTime);
                    int afterDays = DateUtil.getBetWeenDays(divideTime,nextSxrq);
                    double beforeRestTime = VocationCommonUtil.computeRestDays(beforeDays, yearDays, beginEdktsc);
                    double afterRestTime = VocationCommonUtil.computeRestDays(afterDays, yearDays, endEdktsc);
                    restTime = VocationCommonUtil.handleRestDays(beforeRestTime+afterRestTime,wscl);
                }
            }
        }else {
            //不折算
            int beginYear = 0;
            if (jgscsf.equals("2")){
                //直接取工龄字段
                beginYear = Double.valueOf(ljcrglyfslwz).intValue();
            }else {
                //起算日期与当前日期间隔+工龄字段
                int beginYearMonth = DateUtil.getBetWeenMonths(qsrq,sxrq);
                if (beginYearMonth < 0){
                    beginYearMonth = 0;
                }
                beginYearMonth =  beginYearMonth+seniority;
                beginYear = beginYearMonth/12;
            }
            Map<String,Object> beginYearmap = VocationCommonUtil.getVocationMap(vocationList, beginYear, yjzd,userId);
            double beginEdktsc = Double.valueOf(beginYearmap.get("edktsc") == null?  "0" :Util.null2String(beginYearmap.get("edktsc")));
            restTime=beginEdktsc;
        }
        return restTime;
    }

}
