package com.engine.jucailinkq.attendance.workflow.proxy.tactics;

import com.alibaba.fastjson.JSONObject;
import com.engine.jucailinkq.attendance.workflow.enums.WorkFlowTypeEnum;
import com.engine.jucailinkq.common.util.CommonUtil;
import com.engine.jucailinkq.common.util.DateUtil;
import com.engine.jucailinkq.common.util.DbTools;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import lombok.extern.slf4j.Slf4j;

import java.time.ZoneOffset;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 出差流程删除、退回等进行出差记录数据删除
 * @author sy
 */
@Slf4j
public class EvectionDataReset implements WorkFlowHandleTacis{
    @Override
    public boolean support(String personClassfiy) {
        if (personClassfiy.equals(WorkFlowTypeEnum.EVECTION.getKey())){
            return true;
        }
        return false;
    }

    @Override
    public void reset(Map<String, Object> requestMap) {
        log.info("EvectionDataReset requestMap: [{}]",requestMap);
        String tableName = requestMap.get("lcbd").toString();
        String requestId = requestMap.get("requestId").toString();
        String operate = requestMap.get("operate").toString();
        String workflowid = requestMap.get("workflowid").toString();

        Map<String,Object> mainMap = null;
        List<Map<String, Object>> detailTableData = null;

        String detail1TableName = CommonUtil.getDetailTableInfoTableName(tableName+"_dt1",0,WorkFlowTypeEnum.EVECTION,workflowid);

        if (operate.equals("dodelete")){
            //删除流程
            String sql = "select deletetabledata from workflow_requestdeletelog where request_id=? and (isvalid is null or  isvalid <> '1')";
            List<Map<String, Object>> deleteDataList = DbTools.getSqlToList(sql,requestId);
            if (deleteDataList.size() > 0){
                String deletetabledata = deleteDataList.get(0).get("deletetabledata").toString();
                Map<String, List<Map<String, Object>>> map = (Map<String, List<Map<String, Object>>>) JSONObject.parse(deletetabledata);
                mainMap = keyToLowerMap(map.get(tableName).get(0));
                detailTableData = keyToLowerList(map.get(tableName+"_dt1"));
            }
        }else {
            String sql = "select * from "+tableName+" where requestid=?";
            mainMap = DbTools.getSqlToList(sql,requestId).get(0);
            // 流程表单明细表1数据
            sql = "select * from "+detail1TableName+" where mainid=?";
            detailTableData = DbTools.getSqlToList(sql,mainMap.get("id"));
        }
        log.info("detailTableData : [{}]",detailTableData);
        if (mainMap == null || detailTableData == null){
            return;
        }
        try {
            //收集出差记录中流程id关联的数据
            String sql = "select id from uf_jcl_kq_ccjl where djly = 0 and lyid = " + requestId;
            List<Map<String, Object>> ccData = DbTools.getSqlToList(sql);
            if (ccData.size() > 0) {
                List<String> ccjlIdList = ccData.stream().map(f->f.get("id").toString()).collect(Collectors.toList());
                //删除出差记录明细表
                sql = "delete from uf_jcl_kq_ccjl_dt1 where mainid in (" + String.join(",", ccjlIdList) + ")";
                boolean delSign1 = DbTools.update(sql);
                String message = "";
                if (!delSign1) {
                    message = message + "删除出差记录明细表数据失败！";
                }
                //删除出差记录主表
                sql = "delete from uf_jcl_kq_ccjl where id in (" + String.join(",", ccjlIdList) + ")";
                boolean delSign2 = DbTools.update(sql);
                if (!delSign2) {
                    message = message + "删除出差记录主表数据失败！";
                }
                if (!delSign1 || !delSign2) {
                    log.error(message);
                }
                if (detailTableData.size() > 0){
                    Map<String,List<Map<String,Object>>> detailGroupMap = detailTableData.stream().collect(Collectors.groupingBy(e->e.get("ccr").toString()));
                    for (Map.Entry<String,List<Map<String,Object>>> entry: detailGroupMap.entrySet()){
                        String ygid = entry.getKey();
                        List<Map<String,Object>> detailList = entry.getValue();
                        detailList = detailList.stream().sorted(Comparator.comparing(e -> DateUtil.getTime(e.get("ksrq").toString()).toInstant(ZoneOffset.of("+8")).toEpochMilli())).collect(Collectors.toList());
                        String updateSql = "update uf_jcl_kq_cqjg set sjzt=0 where ygid=? and rq>=? and rq<=?";
                        DbTools.update(updateSql,ygid,detailList.get(0).get("ksrq"),detailList.get(detailList.size()-1).get("jsrq"));
                    }
                }
            }
        }catch (Exception e){
            log.error("EvectionDataReset error : [{}]", e.getMessage());
        }
    }

    public Map<String,Object> keyToLowerMap(Map<String,Object> map){
        Map<String,Object> resultMap = Maps.newHashMap();
        map.entrySet().forEach(e->{
            resultMap.put(e.getKey().toLowerCase(),e.getValue());
        });
        return resultMap;
    }
    public List<Map<String,Object>> keyToLowerList(List<Map<String,Object>> list){
        List<Map<String,Object>> resultList = Lists.newArrayList();
        for (Map<String,Object> map:list){
            resultList.add(keyToLowerMap(map));
        }
        return resultList;
    }
}
