package com.engine.jucailinkq.attendance.workflow.proxy.tactics;

import com.alibaba.fastjson.JSONObject;
import com.engine.jucailinkq.attendance.workflow.enums.WorkFlowTypeEnum;
import com.engine.jucailinkq.common.util.DateUtil;
import com.engine.jucailinkq.common.util.DbTools;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import lombok.extern.slf4j.Slf4j;
import weaver.interfaces.workflow.action.Action;

import java.time.ZoneOffset;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 补打卡流程删除、退回等进行补打卡记录数据删除
 * @author sy
 */
@Slf4j
public class MakeUpClockInDataReset implements WorkFlowHandleTacis{
    @Override
    public boolean support(String personClassfiy) {
        if (personClassfiy.equals(WorkFlowTypeEnum.CLOCK_IN.getKey())){
            return true;
        }
        return false;
    }

    @Override
    public void reset(Map<String, Object> requestMap) {
        log.info("MakeUpClockInDataReset requestMap: [{}]",requestMap);
        String tableName = requestMap.get("lcbd").toString();
        String requestId = requestMap.get("requestId").toString();
        String operate = requestMap.get("operate").toString();

        Map<String,Object> mainMap = null;
        List<Map<String, Object>> detailTableData = null;
        if (operate.equals("dodelete")){
            //删除流程
            String sql = "select deletetabledata from workflow_requestdeletelog where request_id=? and (isvalid is null or  isvalid <> '1')";
            List<Map<String, Object>> deleteDataList = DbTools.getSqlToList(sql,requestId);
            if (deleteDataList.size() > 0){
                String deletetabledata = deleteDataList.get(0).get("deletetabledata").toString();
                Map<String, List<Map<String, Object>>> map = (Map<String, List<Map<String, Object>>>) JSONObject.parse(deletetabledata);
                mainMap = keyToLowerMap(map.get(tableName).get(0));
                detailTableData = keyToLowerList(map.get(tableName+"_dt1"));
            }
        }else {
            String sql = "select * from "+tableName+" where requestid=?";
            mainMap = DbTools.getSqlToList(sql,requestId).get(0);
            // 流程表单明细表1数据
            sql = "select * from "+tableName+"_dt1 where mainid=?";
            detailTableData = DbTools.getSqlToList(sql,mainMap.get("id"));
        }
        log.info("detailTableData : [{}]",detailTableData);
        if (mainMap == null || detailTableData == null){
            return;
        }
        try {
            //收集补打卡记录中流程id关联的数据
            String sql = "select id,bdkry,dkrq from uf_jcl_kq_bdkjl where sjly = 0 and lcid = " + requestId;
            List<Map<String, Object>> bdkData = DbTools.getSqlToList(sql);
            if (bdkData.size() > 0) {
                List<String> bdkjlIdList = bdkData.stream().map(f->f.get("id").toString()).collect(Collectors.toList());
                //删除出差记录主表
                sql = "delete from uf_jcl_kq_bdkjl where id in (" + String.join(",", bdkjlIdList) + ")";
                boolean delSign = DbTools.update(sql);
                String message = "";
                if (!delSign) {
                    message = message + "删除补打卡记录表数据失败！";
                    log.error(message);
                }


                Map<String,List<Map<String,Object>>> detailGroupMap = bdkData.stream().collect(Collectors.groupingBy(e->e.get("bdkry").toString()));
                for (Map.Entry<String,List<Map<String,Object>>> entry: detailGroupMap.entrySet()){
                    String ygid = entry.getKey();
                    List<Map<String,Object>> detailList = entry.getValue();
                    detailList = detailList.stream().sorted(Comparator.comparing(e -> DateUtil.getTime(e.get("dkrq").toString()).toInstant(ZoneOffset.of("+8")).toEpochMilli())).collect(Collectors.toList());
                    String updateSql = "update uf_jcl_kq_cqjg set sjzt=0 where ygid=? and rq>=? and rq<=?";
                    log.info("MakeUpClockInDataReset detailList:[{}]",detailList);
                    DbTools.update(updateSql,ygid,detailList.get(0).get("dkrq"),detailList.get(detailList.size()-1).get("dkrq"));
                }

            }
        }catch (Exception e){
            log.error("MakeUpClockInDataReset error : [{}]", e.getMessage());
        }
    }

    public Map<String,Object> keyToLowerMap(Map<String,Object> map){
        Map<String,Object> resultMap = Maps.newHashMap();
        map.entrySet().forEach(e->{
            resultMap.put(e.getKey().toLowerCase(),e.getValue());
        });
        return resultMap;
    }
    public List<Map<String,Object>> keyToLowerList(List<Map<String,Object>> list){
        List<Map<String,Object>> resultList = Lists.newArrayList();
        for (Map<String,Object> map:list){
            resultList.add(keyToLowerMap(map));
        }
        return resultList;
    }
}
