package com.engine.jucailinkq.attendance.workflow.service.impl;

import com.engine.jucailinkq.attendance.component.persongroup.commonutil.PersongroupCommonUtil;
import com.engine.jucailinkq.attendance.component.persongroup.service.SchedulingResultsService;
import com.engine.jucailinkq.attendance.component.persongroup.service.impl.SchedulingResultsServiceImpl;
import com.engine.jucailinkq.attendance.enums.*;
import com.engine.jucailinkq.attendance.workflow.cmd.GetAskForLeaveRecordListCmd;
import com.engine.jucailinkq.attendance.workflow.cmd.GetRestDayIntervalCmd;
import com.engine.jucailinkq.attendance.workflow.enums.AskAndEvctionWayEnum;
import com.engine.jucailinkq.attendance.workflow.service.AskForLeaveService;
import com.engine.jucailinkq.attendance.workflow.service.MakeUpClockInService;
import com.engine.jucailinkq.common.exception.AttendanceRunTimeException;
import com.engine.common.util.*;
import com.engine.core.impl.Service;
import com.engine.jucailinkq.common.util.*;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import lombok.extern.slf4j.Slf4j;
import weaver.general.Util;

import java.time.ZoneOffset;
import java.util.*;
import java.util.stream.Collectors;

@Slf4j
public class AskForLeaveServiceImpl extends Service implements AskForLeaveService {
    private SchedulingResultsService schedulingResultsService = ServiceUtil.getService(SchedulingResultsServiceImpl.class);

    private MakeUpClockInService makeUpClockInService = ServiceUtil.getService(MakeUpClockInServiceImpl.class);

    @Override
    public Map<String, Object> getUesdHolidayItem(Map<String, Object> params) {
        Map<String,Object> resultMap = Maps.newHashMap();
        // 流程表单主表数据
        Map<String,String> mainTableData = (Map<String,String>)params.get("mainTableData");
        // 流程表单明细表1数据
        List<Map<String, String>> detailTableData = (List<Map<String, String>>)params.get("detailTableData");


        if (detailTableData == null || detailTableData.size() == 0){
            return Maps.newHashMap();
        }
        detailTableData = detailTableData.stream().sorted(Comparator.comparing(e-> DateUtil.getTime(e.get("ksrq")).toInstant(ZoneOffset.of("+8")).toEpochMilli())).collect(Collectors.toList());


        //第一笔开始时间
        String firstStartDate = detailTableData.get(0).get("ksrq");

        //请假人员
        String qjry = mainTableData.get("qjry");
        //假期类型
        String jqlx = mainTableData.get("jqlx");

        String sql = "select id,mc,hsdw,hsl,jcbyxsyqjb,yxsydjb,qzsyyxjb from uf_jcl_kq_kqxm where id=?";

        Map<String,Object> holidayItem = DbTools.getSqlToMap(sql,jqlx);


        if (CheckBoxEnum.CHECKED.getKey().equals(holidayItem.get("qzsyyxjb")) &&
                CheckBoxEnum.CHECKED.getKey().equals(holidayItem.get("jcbyxsyqjb"))){
            Map<String,Object> holidayPriorityItem = DbTools.getSqlToMap(sql,holidayItem.get("yxsydjb"));

            sql = "select id,jqid,sxrq,ktsc,yxsc,wxsc,yqsxrq,ztsc from uf_jcl_kq_jqye where ygid=? and jqid=? and sxrq<=? and yqsxrq>=? order by sxrq";

            List<Map<String,Object>> holidayBalanceList = DbTools.getSqlToList(sql,qjry,holidayPriorityItem.get("id"),firstStartDate,firstStartDate);

            Map<String,Object> useItemMap = Maps.newHashMap();
            for (Map<String, String> detail:detailTableData){
                String ksrq = detail.get("ksrq");
                List<Map<String,Object>> list = holidayBalanceList.stream().filter(e-> DateUtil.getTime(e.get("yqsxrq").toString()).compareTo(DateUtil.getTime(ksrq))>=0).collect(Collectors.toList());
                log.debug("ksrq : [{}], list : [{}]",ksrq,list);
                double totalUnearnedLeave = list.stream().mapToDouble(e->Double.valueOf(e.get("wxsc").toString())).sum();
                if (totalUnearnedLeave == 0){
                    List<Map<String,String>> details = (List<Map<String,String>>)useItemMap.get(holidayItem.get("id"));
                    if (details == null){
                        details = Lists.newArrayList();
                    }
                    details.add(detail);
                    useItemMap.put(holidayItem.get("id").toString(),details);
                }else {
                    List<Map<String,String>> details = (List<Map<String,String>>)useItemMap.get(holidayPriorityItem.get("id"));
                    if (details == null){
                        details = Lists.newArrayList();
                    }
                    details.add(detail);
                    useItemMap.put(holidayPriorityItem.get("id").toString(),details);
                }
            }
            //需要使用到2种以上假期
            if (useItemMap.size() > 1){
                throw new AttendanceRunTimeException("优先使用的"+holidayPriorityItem.get("mc")+"余额不足");
            }else {
                if (useItemMap.get(holidayPriorityItem.get("id")) != null){
                    resultMap.put("jqlx",holidayPriorityItem.get("id"));
                    resultMap.put("hsdw",holidayPriorityItem.get("hsdw"));
                    resultMap.put("hsl",holidayPriorityItem.get("hsl"));
                    resultMap.put("mc",holidayPriorityItem.get("mc"));
                    resultMap.put("jcbyxsyqjb",CheckBoxEnum.CHECKED.getKey());
                }else if (useItemMap.get(holidayItem.get("id")) != null){
                    resultMap.put("jqlx",jqlx);
                    resultMap.put("hsdw",holidayItem.get("hsdw"));
                    resultMap.put("hsl",holidayItem.get("hsl"));
                    resultMap.put("mc",holidayItem.get("mc"));
                    resultMap.put("jcbyxsyqjb",CheckBoxEnum.UNCHECKED.getKey());
                }
            }
        }else {
            resultMap.put("jqlx",jqlx);
            resultMap.put("hsdw",holidayItem.get("hsdw"));
            resultMap.put("hsl",holidayItem.get("hsl"));
            resultMap.put("mc",holidayItem.get("mc"));
            resultMap.put("jcbyxsyqjb",CheckBoxEnum.UNCHECKED.getKey());
        }

        return resultMap;
    }

    @Override
    public Map<String, Object> getAskForLeaveRecordList(Map<String, Object> params) {
        return commandExecutor.execute(new GetAskForLeaveRecordListCmd(params,user));
    }

    @Override
    public Map<String, Object> getRestDayInterval(Map<String, Object> params) {
        Map<String, Object> resultMap = Maps.newHashMap();
        String itemId = Util.null2String(params.get("itemId"));
        String sql = "select zdycrqqjndxxb from uf_jcl_kq_kqxm where id=?";
        List<Map<String,Object>> itemList = DbTools.getSqlToList(sql,itemId);
        String zdycrqqjndxxb = Util.null2String(itemList.get(0).get("zdycrqqjndxxb"));
        if (CheckBoxEnum.UNCHECKED.getKey().equals(zdycrqqjndxxb)){
            return resultMap;
        }
        return commandExecutor.execute(new GetRestDayIntervalCmd(params,user));
    }

    /**
     * 批量请假-生成请假信息明细表数据
     */
    @Override
    public Map<String, Object> generateLeaveInfoList(Map<String, Object> params){
        Map<String,Object> resultMap = new HashMap<>();
        try {
            List<String> errorMessage = new ArrayList<>();
            // 流程表单主表数据
            Map<String,String> mainTableData = (Map<String,String>)params.get("mainTableData");
            log.info("mainTableData : {}", mainTableData);
            // 流程表单明细表1数据
            List<Map<String, String>> detailTableData = (List<Map<String, String>>)params.get("detailTableData");
            String checkCycle = Util.null2String(params.get("checkCycle"));
            //需要校验假期额度的假期类型
            List<String> checkAmountJqIdList = CommonUtil.getJqInfoWithAmount();
            //开始日期、结束日期、请假方式、开始时间、结束时间、假期类型、请假时长
            String startDate = Util.null2String(mainTableData.get("ksrq"));
            String endDate = Util.null2String(mainTableData.get("jsrq"));
            String leaveMode = Util.null2String(mainTableData.get("cxjqj"));
            String leaveType = Util.null2String(mainTableData.get("jqlx"));
            String dailyRepeat = Util.null2String(mainTableData.get("mtcfsdjq"));
            //假期余额使用规则，0-假别优先，1-失效日期优先，默认0
            String jqyeUsePriority = Util.null2String(mainTableData.get("yesygz"));
            if ("".equals(jqyeUsePriority)) {
                jqyeUsePriority = "1";
            }
            //获取请假日期集合
            List<String> leaveDateList = DateUtil.getDatesBetween(startDate, endDate);
            //已编辑内容中的假期额度使用信息
            Map<String, Double> editedUseJqed = new HashMap<>();
            //记录本次触发按钮时，目标日期区间内已编辑内容中的<假期类型_人员,使用时长>信息
            Map<String, Double> editedUseJqlxWithEmp = new HashMap<>();
            //假期余额时长已用完的假期余额id
            List<String> unableUseJqyeIdList = new ArrayList<>();
            //记录已编辑的请假记录，<人员id-请假类型-日期, 请假时长>
            Map<String, String> editedLeaveInfo = new HashMap<>();
            //记录已编辑的人员请假记录，<人员id, 请假信息list>
            Map<String, List<Map<String, String>>> editedEmpLeaveInfo = new HashMap<>();
            //20241029需求变更，增加单天请假“全局唯一性”约束规则，此处记录已编辑的人员相关的请假信息
            editedEmpLeaveInfo = detailTableData == null ? new HashMap<>() : detailTableData.stream().collect(Collectors.groupingBy(map -> map.get("qjr")));
            //处理明细表1数据，生成触发接口之前编辑内容中已使用的假期额度信息
            editedLeaveInfo = dealDetailTableData(detailTableData, leaveDateList, editedUseJqlxWithEmp, editedUseJqed, unableUseJqyeIdList, editedLeaveInfo);
            //处理主表数据
            String qjr = Util.null2String(mainTableData.get("qjr"));
            //请假人姓名映射
            Map<String, String> empIdToName = new HashMap<>();
            //请假人员列表
            List<String> leaveEmpIdList = new ArrayList<>();
            if (!"".equals(qjr)) {
                leaveEmpIdList = Arrays.asList(qjr.split(","));
                String sql = "select id, lastname from hrmresource where id in (" + qjr + ")";
                List<Map<String, Object>> data = DbTools.getSqlToList(sql);
                empIdToName = data.stream().collect(Collectors.toMap(e->Util.null2String(e.get("id")),e->Util.null2String(e.get("lastname"))));
            }
            //获取填写的请假类型关联的考勤项目
            String sql = "select id,mc,hsdw,hsl,jcbyxsyqjb,yxsydjb,qzsyyxjb,zdycbcndfgzsd,zdycrqqjndxxb,yxyz,zdyzsl,zysd from uf_jcl_kq_kqxm where id=?";
            Map<String,Object> holidayItem = DbTools.getSqlToMap(sql,leaveType);
            //勾选假期类型名称
//            String checkItemName = Util.null2String(holidayItem.get("mc"));
            //核算单位
            String hsdw = Util.null2String(holidayItem.get("hsdw"));
            //核算量
            double hsl = Double.valueOf(Util.null2String(holidayItem.get("hsl")));
            //核算单位为天时，只允许请全天假、半天假
            if (hsdw.equals(AccountingUnitEnum.DAY.getKey()) && !leaveMode.equals(AskAndEvctionWayEnum.ALLDAY.getKey()) && !leaveMode.equals(AskAndEvctionWayEnum.HALFDAY.getKey())) {
                resultMap.put("status", false);
                resultMap.put("errorInfo", "假期类型的核算单位为天时，只允许请全天假、半天假！");
                resultMap.put("data", null);
                return resultMap;
            }
            //判断是否需要自动移除时间区间内的非工作时长、自动移除日期区间内的休息日
            boolean removeNonWorkTimeRange = "1".equals(Util.null2String(holidayItem.get("zdycbcndfgzsd")));
            boolean removeNonWorkDayRange = "1".equals(Util.null2String(holidayItem.get("zdycrqqjndxxb")));
            //获取作用时段
            String zysd = Util.null2String(holidayItem.get("zysd"));
            //获取需要统计时长的班段类型集合（仅作用在“指定时间区间”请假方式）
            List<String> countBdlxList = new ArrayList<>();
            List<String> zysdList = new ArrayList<>();
            if (!"".equals(zysd)) {
                zysdList = Arrays.asList(zysd.split(","));
                for(String zysdKey : zysdList) {
                    countBdlxList.add(Utils.getClassSegmenByWorkFor(zysdKey));
                }
                if (!removeNonWorkTimeRange) {
                    countBdlxList.add(ClassSegmentTypeEnum.REST_AND_DINE.getKey());
                    countBdlxList.add(ClassSegmentTypeEnum.REST_PERIOD.getKey());
                    countBdlxList.add(ClassSegmentTypeEnum.DINING_PERIOD.getKey());
                }
            }
            //20241021逻辑变更，根据考勤项目的“优先使用项目”信息，逐级找到所有“优先使用项目”，找到的先后顺序，来决定使用优先级，最后的优先级最大
            List<Map<String, Object>> holidayPriorityItemList = new ArrayList<>();
            holidayPriorityItemList.add(holidayItem);
            //获取假期类型的考勤项目
            sql = "select id,mc,hsdw,hsl,jcbyxsyqjb,yxsydjb,qzsyyxjb,zdycbcndfgzsd,zdycrqqjndxxb,yxyz,zdyzsl,zysd from uf_jcl_kq_kqxm where xmlx=? or xmlx=?";
            List<Map<String,Object>> holidayItemList = DbTools.getSqlToList(sql, AttendanceItemTypeEnum.HOLIDAY.getKey(), AttendanceItemTypeEnum.EVECTION.getKey());
            Map<String, Map<String,Object>> holidayItemInfos = holidayItemList.stream().collect(Collectors.toMap(e->e.get("id").toString(), e->e));
            Map<String, String> jbIdToHsdw = holidayItemList.stream().collect(Collectors.toMap(e->e.get("id").toString(), e->e.get("hsdw").toString()));
            //逐级获取优先使用项目
            getHolidayPriorityItem(holidayPriorityItemList, holidayItemInfos, holidayItem);
            //20241118需求变更，“指定时间区间”、“指定时长”方式请假时，不允许使用核算单位为“天”的假别
            if (leaveMode.equals(AskAndEvctionWayEnum.TIME_INTERVAL.getKey()) || leaveMode.equals(AskAndEvctionWayEnum.HOUR.getKey())) {
                holidayPriorityItemList = holidayPriorityItemList.stream().filter(f -> !f.get("hsdw").toString().equals(AccountingUnitEnum.DAY.getKey())).collect(Collectors.toList());
            }
            //遍历人员、日期，生成人员+日期+请假时长的请假信息
            //请假人的请假区间内每一天的日期类型信息
            Map<String, List<String>> restDayInfo = removeNonWorkDayRange ? getRestDayWithEmpId(leaveEmpIdList, leaveType, startDate, endDate) : new HashMap<>();
            //请假人的请假区间内的排班结果
            Map<String, List<Map<String, Object>>> scheduleInfoMap = getScheduleInfoWithEmpId(leaveEmpIdList, DateUtil.beforeDay(startDate,1), DateUtil.AfterDay(endDate,1));
            //班次信息与班次id的映射
            Map<String, Map<String, Object>> shiftInfoMap = getShiftInfoWithShiftId();
            //收集未关联假期余额的请假明细数据
            List<Map<String, String>> simpleLeaveDetailList;
            //收集请假人有班次时的日期对应的额定时长信息
            Map<String, Map<String, Double>> empIdToDateRatedHours = new HashMap<>();
            if (leaveMode.equals(AskAndEvctionWayEnum.TIME_INTERVAL.getKey()) && !"1".equals(dailyRepeat)) {
                simpleLeaveDetailList = createSimpleLeaveDetailListWithNoDaily(leaveEmpIdList, leaveDateList, removeNonWorkDayRange, scheduleInfoMap,
                        restDayInfo, countBdlxList, hsdw, hsl, shiftInfoMap, mainTableData, empIdToDateRatedHours);
            } else {
                simpleLeaveDetailList = createSimpleLeaveDetailList(leaveEmpIdList, leaveDateList, removeNonWorkDayRange, scheduleInfoMap,
                        restDayInfo, countBdlxList, hsdw, hsl, shiftInfoMap, mainTableData, empIdToDateRatedHours);
            }
            //收集已关联假期余额的请假明细数据
            List<Map<String, String>> completeLeaveDetailList = new ArrayList<>();
            //按照人员id分组处理请假明细信息，关联假期余额数据
            Map<String, List<Map<String, String>>> leaveDetailGroupByEmp = simpleLeaveDetailList.stream().collect(Collectors.groupingBy(e -> Util.null2String(e.get("qjr"))));
            //获取modeId，假期额度规则的modeId
            Map<String,String> formmodeIdMap = Utils.getFormmodeIdMap();
            String modeId = formmodeIdMap.get("uf_jcl_kq_jqed");
            for(Map.Entry<String,List<Map<String,String>>> entry : leaveDetailGroupByEmp.entrySet()) {
                //校验该人员的考勤周期是否正常
                if (!checkCycle.equals("0")){
                    //需要检验
                    boolean kqCycleAllow = kqCycleCheck(entry.getKey(), entry.getValue(), errorMessage, empIdToName.get(entry.getKey()));
                    if (!kqCycleAllow) {
                        continue;
                    }
                }
                //20241021逻辑变更，根据考勤项目的“优先使用项目”信息，获取所有可使用假期余额信息
                Map<String, List<Map<String, Object>>> canUseJqyeInfo = new HashMap<>();
                //20241107需求变更，获取所有可使用假期余额信息列表，并按照失效期+级联优先级排序
                List<Map<String, Object>> canUseJqyeList = new ArrayList<>();
                //记录假别优先级
                int jbPriorityValue = 1;
                List<Map<String, Object>> canUseJqyeItem;
                for (Map<String, Object> item : holidayPriorityItemList) {
                    if (!checkAmountJqIdList.contains(item.get("id").toString())) {
                        continue;
                    }
                    //20241107需求变更，根据流程中设置的假期额度使用优先规则，收集假期余额数据
                    canUseJqyeItem = collectUsableHolidayBalance(unableUseJqyeIdList, editedUseJqed, item, startDate, entry.getKey(), empIdToName.get(entry.getKey()), modeId, errorMessage, jbPriorityValue);
                    jbPriorityValue++;
                    if ("0".equals(jqyeUsePriority)) {
                        canUseJqyeInfo.put(item.get("id").toString(), canUseJqyeItem);
                    } else if ("1".equals(jqyeUsePriority)) {
                        canUseJqyeList.addAll(canUseJqyeItem);
                    }
                }
                if (canUseJqyeList.size() > 0) {
                    //按照延期失效日期早+级联优先级高（优先数字小）规则排序
                    canUseJqyeList.sort(Comparator.comparing((Map<String, Object> h) -> (String) h.get("yqsxrq")).thenComparing(h->h.get("sxrq").toString())
                            .thenComparing(h -> (Integer) h.get("jbPriorityValue")));
                }

                List<Map<String,String>> detailListItem = new ArrayList<>();
                if ("0".equals(jqyeUsePriority)) {
                    detailListItem = matchHolidayWithPriority(editedLeaveInfo, checkAmountJqIdList, errorMessage, entry.getKey(),
                            empIdToName.get(entry.getKey()), canUseJqyeInfo, holidayPriorityItemList, editedUseJqed, editedUseJqlxWithEmp,
                            entry.getValue(), editedEmpLeaveInfo.get(entry.getKey()), jbIdToHsdw, empIdToDateRatedHours.get(entry.getKey()));
                } else if ("1".equals(jqyeUsePriority)) {
                    detailListItem =  matchHolidayWithDatePriority(editedLeaveInfo, checkAmountJqIdList, errorMessage, entry.getKey(),
                            empIdToName.get(entry.getKey()), canUseJqyeList, holidayPriorityItemList, editedUseJqed, editedUseJqlxWithEmp,
                            entry.getValue(), editedEmpLeaveInfo.get(entry.getKey()), jbIdToHsdw, empIdToDateRatedHours.get(entry.getKey()));
                }
                if (detailListItem.size() > 0) {
                    Map<String,List<Map<String,String>>> detailGroup = detailListItem.stream().collect(Collectors.groupingBy(e->Util.null2String(e.get("ksrq"))+"&"+Util.null2String(e.get("jsrq"))));
                    List<Map<String,String>> newDetailListItem = Lists.newArrayList();
                    for (Map.Entry<String,List<Map<String,String>>> detailEntry :detailGroup.entrySet()){
                        String date = detailEntry.getKey();
                        List<Map<String,String>> detailList = detailEntry.getValue();
                        if (detailList.size() > 1){
                            String jqye = "";
                            double qjsc = 0;
                            for (Map<String,String> map:detailList){
                                if (!Util.null2String(map.get("jqye")).equals("")){
                                    jqye += map.get("jqye")+",";
                                }
                                qjsc +=Double.valueOf(map.get("qjsc"));
                            }
                            if (!jqye.equals("")){
                                jqye = jqye.substring(0,jqye.length()-1);
                            }
                            detailList.get(0).put("jqye",jqye);
                            detailList.get(0).put("qjsc",Util.null2String(qjsc));
                            detailList.get(0).put("qjlx",holidayPriorityItemList.get(holidayPriorityItemList.size()-1).get("id").toString());
                            detailList.get(0).put("qjlxName",holidayPriorityItemList.get(holidayPriorityItemList.size()-1).get("mc").toString());
                            newDetailListItem.add(detailList.get(0));
                        }else {
                            newDetailListItem.addAll(detailList);
                        }
                    }

                    completeLeaveDetailList.addAll(newDetailListItem);
                }
            }

            if (errorMessage.size() == 0) {
                resultMap.put("status", true);
                resultMap.put("data", completeLeaveDetailList);
            } else {
                resultMap.put("status", false);
                resultMap.put("errorInfo", errorMessage);
                resultMap.put("data", null);
            }

        } catch (Exception e) {
            log.info(e.getMessage());
            resultMap.put("status", false);
            resultMap.put("errorInfo", e.getMessage());
            resultMap.put("data", null);
        }
        return resultMap;
    }

    private Map<String, Map<String, Object>> getShiftInfoWithShiftId() {
        Map<String, Map<String, Object>> result = new HashMap<>();
        String sql = "select id, mc ,edsc, fgsjd, zgzsc, btgz from uf_jcl_kq_bcxx";
        List<Map<String, Object>> shiftInfoList = DbTools.getSqlToList(sql);
        if (shiftInfoList.size() > 0) {
            result = shiftInfoList.stream().collect(Collectors.toMap(e -> e.get("id").toString(), e -> e));
        }
        return result;
    }

    private void getHolidayPriorityItem(List<Map<String, Object>> holidayPriorityItemList, Map<String, Map<String, Object>> holidayItemInfos, Map<String, Object> holidayItem) {
        boolean useFirstItemSign = CheckBoxEnum.CHECKED.getKey().equals(holidayItem.get("qzsyyxjb")) && CheckBoxEnum.CHECKED.getKey().equals(holidayItem.get("jcbyxsyqjb"));
        String useFirstItemId = useFirstItemSign ? Util.null2String(holidayItem.get("yxsydjb")) : "";
        if (!"".equals(useFirstItemId) && holidayItemInfos.get(useFirstItemId) != null) {
            holidayPriorityItemList.add(0, holidayItemInfos.get(useFirstItemId));
            //获取下一级优先使用项目
            getHolidayPriorityItem(holidayPriorityItemList, holidayItemInfos, holidayItemInfos.get(useFirstItemId));
        }
    }

    /**
     * @param detailTableData 明细表1数据
     * @param leaveDateList 请假日期集合
     * @param editedUseJqlxWithEmp 记录本次触发按钮时，目标日期区间内已编辑内容中的<假期类型_人员,使用时长>信息
     * @param editedUseJqed 记录已编辑内容中的假期额度使用信息
     * @param unableUseJqyeIdList 记录假期余额时长已用完的假期余额id
     * @param editedLeaveInfo 记录记录已编辑的请假记录，<人员id-请假类型-日期, 请假时长>
     * @return 处理明细表1数据，生成触发接口之前编辑内容中已使用的假期额度信息
     */
    private Map<String, String> dealDetailTableData(List<Map<String, String>> detailTableData, List<String> leaveDateList, Map<String, Double> editedUseJqlxWithEmp,
                                                    Map<String, Double> editedUseJqed, List<String> unableUseJqyeIdList, Map<String, String> editedLeaveInfo) {
        if (detailTableData != null && detailTableData.size() > 0){
            for (Map<String, String> detailItem : detailTableData) {
                if (leaveDateList.contains(Util.null2String(detailItem.get("ksrq")))) {
                    double detailSc = "".equals(Util.null2String(detailItem.get("qjsc"))) ? 0 : Double.parseDouble(Util.null2String(detailItem.get("qjsc")));
                    String qjlx = Util.null2String(detailItem.get("qjlx"));
                    String empId = Util.null2String(detailItem.get("qjr"));
                    editedUseJqlxWithEmp.merge(qjlx + "_" + empId, detailSc, Double::sum);
                }
                String jqye = Util.null2String(detailItem.get("jqye"));
                if ("".equals(jqye)) {
                    continue;
                } else if (jqye.contains(",")) {
                    List<String> jqyeIds = Arrays.asList(jqye.split(","));
                    for (int i = 0; i < jqyeIds.size(); i++ ) {
                        //假期余额id会用下划线“_”拼接上该假期余额记录被使用的时长
                        String jqyeIdStr = jqyeIds.get(i);
                        String[] jqyeInfo = jqyeIdStr.split("_");
                        String jqyeId = jqyeInfo[0];
                        double qjsc = Double.parseDouble(jqyeInfo[1]);
                        editedUseJqed.merge(jqyeId, qjsc, Double::sum);
                        //最后一个id前的jqye的id都属于额度已使用完的，记录到已用完假期余额id集合中
                        if (i < jqyeIds.size() - 1) {
                            unableUseJqyeIdList.add(jqyeId);
                        }
                    }
                } else {
                    String[] jqyeInfo = jqye.split("_");
                    String jqyeId = jqyeInfo[0];
                    double qjsc = Double.parseDouble(jqyeInfo[1]);
                    editedUseJqed.merge(jqyeId, qjsc, Double::sum);
                }
            }
            editedLeaveInfo = detailTableData.stream()
                    .collect(Collectors.toMap(e-> Util.null2String(e.get("qjr")) + "_" + Util.null2String(e.get("qjlx")) + "_" + Util.null2String(e.get("ksrq")),
                            e->Util.null2String(e.get("qjsc"))));
        }
        return editedLeaveInfo;
    }

    /**
     * @param leaveEmpIdList 请假人员id集合
     * @param leaveDateList 请假日期集合
     * @param removeNonWorkDayRange 是否移除非工作日时长
     * @param scheduleInfoMap 排班信息
     * @param restDayInfo 休息日信息
     * @param countBdlxList 收集需要统计时长的班段类型
     * @return 组装初步的请假明细
     */
    private List<Map<String, String>> createSimpleLeaveDetailList(List<String> leaveEmpIdList, List<String> leaveDateList, boolean removeNonWorkDayRange, Map<String, List<Map<String, Object>>> scheduleInfoMap,
                                                                  Map<String, List<String>> restDayInfo, List<String> countBdlxList, String hsdw, double hsl, Map<String, Map<String, Object>> shiftInfoMap,
                                                                  Map<String,String> mainTableData, Map<String, Map<String, Double>> empIdToDateRatedHours) {
        //开始时间、结束时间、请假方式、请假时长、上下午选择、开始时段、结束时段
        String startTime = Util.null2String(mainTableData.get("kssj"));
        String endTime = Util.null2String(mainTableData.get("jssj"));
        String leaveMode = Util.null2String(mainTableData.get("cxjqj"));
        String leaveDuration = Util.null2String(mainTableData.get("qjsc"));
        String halfDayRangeSelect = Util.null2String(mainTableData.get("sxwxz"));
        String startRange = Util.null2String(mainTableData.get("kssd"));
        String endRange= Util.null2String(mainTableData.get("jssd"));

        List<Map<String, String>> simpleLeaveDetailList = new ArrayList<>();
        Map<String, String> simpleLeaveDetailItem;
        String qjscHours = "";
        String sql = "";
        Map<String, Double> dateToRatedHours;
        for (String leaveEmpId : leaveEmpIdList) {
            dateToRatedHours = new HashMap<>();
            //需要自动移除日期区间内的休息日时，去除请假日期区间中的休息日
            List<Map<String, Object>> scheduleInfoList = scheduleInfoMap.getOrDefault(leaveEmpId, new ArrayList<>());
            Map<String, String> dateToBcxxMap = scheduleInfoList.stream().collect(Collectors.toMap(e->Util.null2String(e.get("bcrq")),e->Util.null2String(e.get("bcxx"))));

            List<String> restDateList = new ArrayList<>();
            if (removeNonWorkDayRange) {
                //排班结果中休息的日期
                List<String> restDateListFromSchedule = scheduleInfoList.stream()
                        .filter(f -> Util.null2String(f.get("sfxx")).equals(CheckBoxEnum.CHECKED.getKey()))
                        .map(e -> Util.null2String(e.get("bcrq")))
                        .collect(Collectors.toList());
                List<String> allDateListFromSchedule = scheduleInfoList.stream().map(e -> Util.null2String(e.get("bcrq"))).collect(Collectors.toList());
                //排班结果中未出现的日期
                List<String> nonSetDateListFromSchedule = leaveDateList.stream().filter(f -> !allDateListFromSchedule.contains(f)).collect(Collectors.toList());
                //企业日历中人员对应日期的休息日
                restDateList = restDayInfo.get(leaveEmpId);
                //筛选排班信息无法辨别的日期，依靠企业日历的日期信息
                restDateList = restDateList.stream().filter(nonSetDateListFromSchedule::contains).collect(Collectors.toList());
                restDateList.addAll(restDateListFromSchedule);
            }
            for (String leaveDate : leaveDateList) {
                //获取当天班次id
                String currentDayBcId = restDateList.contains(leaveDate) ? "" : Util.null2String(dateToBcxxMap.get(leaveDate)).split("-")[0];
                if (!"".equals(currentDayBcId)) {
                    Map<String, Object> shiftInfo = shiftInfoMap.get(currentDayBcId);
                    qjscHours = shiftInfo == null ? "8" : shiftInfo.get("edsc").toString();
                    dateToRatedHours.put(leaveDate, Double.parseDouble(qjscHours));
                }
                //请假方式为“指定时间区间”时
                if (leaveMode.equals(AskAndEvctionWayEnum.TIME_INTERVAL.getKey())) {
                    simpleLeaveDetailItem = new HashMap<>();
                    int scMinutes = 0;
                    String realStartDate = leaveDate;
                    String realStartTime = startTime;
                    String realEndDate = leaveDate;
                    String realEndTime = endTime;
                    //组装初步的请假明细数据
                    simpleLeaveDetailItem.put("qjr", leaveEmpId);
                    //获取当天班次id
//                    String currentDayBcId = restDateList.contains(leaveDate) ? "" : Util.null2String(dateToBcxxMap.get(leaveDate)).split("-")[0];
                    //获取前一天班次id
                    String yesterday = DateUtil.beforeDay(leaveDate,1);
                    String yesterdayBcId = restDateList.contains(yesterday) ? "" : Util.null2String(dateToBcxxMap.get(yesterday)).split("-")[0];
                    //获取次日班次id
                    String nextDay = DateUtil.AfterDay(leaveDate,1);
                    String nextDayBcId = restDateList.contains(nextDay) ? "" : Util.null2String(dateToBcxxMap.get(nextDay)).split("-")[0];

                    if (halfDayRangeSelect.equals(CheckBoxEnum.CHECKED.getKey())) {
                        //根据“上下午选择”字段来重新获取开始和结束时间点
                        //获取开始日期班次id
                        if ("".equals(currentDayBcId)) {
                            continue;
                        }
                        //获取开始时间、结束时间
                        String startFromBc = getTimePointFromBc(leaveDate, shiftInfoMap.get(currentDayBcId), "start", startRange);
                        String endFromBc = getTimePointFromBc(leaveDate, shiftInfoMap.get(currentDayBcId),"end", endRange);
                        if ("".equals(startFromBc) || "".equals(endFromBc)) {
                            continue;
                        }
                        realStartDate = startFromBc.split(" ")[0];
                        realStartTime = startFromBc.split(" ")[1];
                        realEndDate = endFromBc.split(" ")[0];
                        realEndTime = endFromBc.split(" ")[1];
                    } else {
                        //开始时时和结束时间存在跨天情况时,即开始时间大于等于结束时间
                        if (startTime.compareTo(endTime) >= 0) {
                            realEndDate = DateUtil.AfterDay(leaveDate,1);
                        }

                    }
                    simpleLeaveDetailItem.put("ksrq", realStartDate);
                    simpleLeaveDetailItem.put("jsrq", realEndDate);
                    simpleLeaveDetailItem.put("kssj", realStartTime);
                    simpleLeaveDetailItem.put("jssj", realEndTime);

                    List<Map<String, Object>> bcDetailData;
                    //开始时时和结束时间存在跨天情况时,即开始时间大于等于结束时间
                    if (startTime.compareTo(endTime) >= 0) {
                        realEndDate = nextDay;
                    }

                    if (!"".equals(yesterdayBcId)) {
                        sql = "select id, bdlx, gsrq, kssj as dtkssj, jssj as dtjssj from uf_jcl_kq_bcxx_dt1 where mainid  = " + yesterdayBcId;
                        bcDetailData = DbTools.getSqlToList(sql);
                        bcDetailData = bcDetailData.stream().filter(e -> countBdlxList.contains(Util.null2String(e.get("bdlx")))).collect(Collectors.toList());
                        //获取需要累计的班段时长区间和请假区间存在交集的分钟数
                        scMinutes = Utils.removeTime(realStartDate + " " + realStartTime, realEndDate + " " + realEndTime, bcDetailData, yesterday);
                    }
                    if (!"".equals(currentDayBcId)) {
                        sql = "select id, bdlx, gsrq, kssj as dtkssj, jssj as dtjssj from uf_jcl_kq_bcxx_dt1 where mainid  = " + currentDayBcId;
                        bcDetailData = DbTools.getSqlToList(sql);
                        bcDetailData = bcDetailData.stream().filter(e -> countBdlxList.contains(Util.null2String(e.get("bdlx")))).collect(Collectors.toList());
                        //获取需要累计的班段时长区间和请假区间存在交集的分钟数
                        scMinutes = scMinutes + Utils.removeTime(realStartDate + " " + realStartTime, realEndDate + " " + realEndTime, bcDetailData, leaveDate);
                    }
                    if (!"".equals(nextDayBcId)) {
                        sql = "select id, bdlx, gsrq, kssj as dtkssj, jssj as dtjssj from uf_jcl_kq_bcxx_dt1 where mainid  = " + nextDayBcId;
                        bcDetailData = DbTools.getSqlToList(sql);
                        bcDetailData = bcDetailData.stream().filter(e -> countBdlxList.contains(Util.null2String(e.get("bdlx")))).collect(Collectors.toList());
                        //获取需要累计的班段时长区间和请假区间存在交集的分钟数
                        scMinutes = scMinutes + Utils.removeTime(realStartDate + " " + realStartTime, realEndDate + " " + realEndTime, bcDetailData, nextDay);
                    }
                    //增加加班计划中的数据，加班计划明细中的数据作为“加班计划”班段类型数据的补充
                    if (countBdlxList.contains(ClassSegmentTypeEnum.OVERTIME_PLAN.getKey())) {
                        sql = "select b.jbry,b.ksrq,b.kssj,b.jblx,b.jsrq,b.jssj,b.jbsc,b.gsrq from uf_jcl_kq_jbjh a left join uf_jcl_kq_jbjh_dt1 b on a.id=b.mainid where b.jbry =? and b.ksrq>=? and b.jsrq<=? and (b.jbcx=0 or b.jbcx is null) and a.jlzt=1";
                        List<Map<String,Object>> overtimePlanList = DbTools.getSqlToList(sql, leaveEmpId, DateUtil.beforeDay(leaveDate,1), DateUtil.AfterDay(realEndDate,1));
                        int scMinutesInOvertimePlan = Utils.removeTimeWithOvertimePlan(realStartDate + " " + realStartTime, realEndDate + " " + realEndTime, overtimePlanList, dateToBcxxMap);
                        scMinutes = scMinutes + scMinutesInOvertimePlan;
                    }
                    if (scMinutes > 0) {
                        double qjscByAccount = Utils.getItemduration(hsl, hsdw, scMinutes, AccountingUnitEnum.MINUTES, 8.0);
                        //组装初步的请假时长
                        simpleLeaveDetailItem.put("qjsc", hsdw.equals(AccountingUnitEnum.MINUTES.getKey())
                                ? String.format("%.2f", qjscByAccount / 60.0)
                                : String.valueOf(qjscByAccount));
                        simpleLeaveDetailList.add(simpleLeaveDetailItem);
                    }
                } else if (leaveMode.equals(AskAndEvctionWayEnum.HOUR.getKey()) && !restDateList.contains(leaveDate)) {
                    simpleLeaveDetailItem = new HashMap<>();
                    //组装初步的请假明细数据
                    simpleLeaveDetailItem.put("qjr", leaveEmpId);
                    simpleLeaveDetailItem.put("ksrq", leaveDate);
                    simpleLeaveDetailItem.put("jsrq", leaveDate);
                    simpleLeaveDetailItem.put("qjsc", leaveDuration);
                    if (Double.parseDouble(leaveDuration) > 0) {
                        double qjscByAccount = Utils.getItemduration(hsl, hsdw, Double.parseDouble(leaveDuration), AccountingUnitEnum.HOUR, 8.0);
                        //组装初步的请假时长
                        simpleLeaveDetailItem.put("qjsc", hsdw.equals(AccountingUnitEnum.MINUTES.getKey())
                                ? String.format("%.2f", qjscByAccount / 60.0)
                                : String.valueOf(qjscByAccount));
                        simpleLeaveDetailList.add(simpleLeaveDetailItem);
                    }
                } else if (leaveMode.equals(AskAndEvctionWayEnum.ALLDAY.getKey()) && !restDateList.contains(leaveDate)) {
                    simpleLeaveDetailItem = new HashMap<>();
                    Map<String, Object> shiftInfo = shiftInfoMap.get(currentDayBcId);
                    qjscHours = shiftInfo == null ? "8" : shiftInfo.get("edsc").toString();
                    //组装初步的请假明细数据
                    simpleLeaveDetailItem.put("qjr", leaveEmpId);
                    simpleLeaveDetailItem.put("ksrq", leaveDate);
                    simpleLeaveDetailItem.put("jsrq", leaveDate);
                    simpleLeaveDetailItem.put("qtj", "1");
                    simpleLeaveDetailItem.put("qjscHours", qjscHours);
                    simpleLeaveDetailList.add(simpleLeaveDetailItem);
                } else if (leaveMode.equals(AskAndEvctionWayEnum.HALFDAY.getKey()) && !restDateList.contains(leaveDate)) {
                    simpleLeaveDetailItem = new HashMap<>();
                    Map<String, Object> shiftInfo = shiftInfoMap.get(currentDayBcId);
                    qjscHours = shiftInfo == null ? "4" : String.valueOf(Double.parseDouble(shiftInfo.get("edsc").toString()) / 2);
                    //组装初步的请假明细数据
                    simpleLeaveDetailItem.put("qjr", leaveEmpId);
                    simpleLeaveDetailItem.put("ksrq", leaveDate);
                    simpleLeaveDetailItem.put("jsrq", leaveDate);
                    simpleLeaveDetailItem.put("btj", "1");
                    simpleLeaveDetailItem.put("qjscHours", qjscHours);
                    simpleLeaveDetailList.add(simpleLeaveDetailItem);
                }
            }
            empIdToDateRatedHours.put(leaveEmpId, dateToRatedHours);
        }
        return simpleLeaveDetailList;
    }

    /**
     * @param leaveEmpIdList 请假人员id集合
     * @param leaveDateList 请假日期集合
     * @param removeNonWorkDayRange 是否移除非工作日时长
     * @param scheduleInfoMap 排班信息
     * @param restDayInfo 休息日信息
     * @param countBdlxList 收集需要统计时长的班段类型
     * @return 组装初步的请假明细，这边只处理“每天重复时段请假”未勾选，且请假方式为“指定时间区间”
     */
    private List<Map<String, String>> createSimpleLeaveDetailListWithNoDaily(List<String> leaveEmpIdList, List<String> leaveDateList, boolean removeNonWorkDayRange, Map<String, List<Map<String, Object>>> scheduleInfoMap,
                                                                             Map<String, List<String>> restDayInfo, List<String> countBdlxList, String hsdw, double hsl, Map<String, Map<String, Object>> shiftInfoMap,
                                                                             Map<String,String> mainTableData, Map<String, Map<String, Double>> empIdToDateRatedHours) {
        //开始日期、结束日期、开始时间、结束时间、请假方式、请假时长、上下午选择、开始时段、结束时段
        String startDate = Util.null2String(mainTableData.get("ksrq"));
        String endDate = Util.null2String(mainTableData.get("jsrq"));
        String startTime = Util.null2String(mainTableData.get("kssj"));
        String endTime = Util.null2String(mainTableData.get("jssj"));
        String halfDayRangeSelect = Util.null2String(mainTableData.get("sxwxz"));
        String startRange = Util.null2String(mainTableData.get("kssd"));
        String endRange = Util.null2String(mainTableData.get("jssd"));

        List<Map<String, String>> simpleLeaveDetailList = new ArrayList<>();
        Map<String, String> simpleLeaveDetailItem;
        String sql = "";
        List<Map<String, Object>> bcDetailData;
        String targetDateBcId = "";
        leaveDateList.add(0, DateUtil.beforeDay(startDate,1));
        leaveDateList.add(DateUtil.AfterDay(endDate,1));
        Map<String, Double> dateToRatedHours;
        for (String leaveEmpId : leaveEmpIdList) {
            dateToRatedHours = new HashMap<>();
            int scMinutes = 0;
            //需要自动移除日期区间内的休息日时，去除请假日期区间中的休息日
            List<Map<String, Object>> scheduleInfoList = scheduleInfoMap.getOrDefault(leaveEmpId, new ArrayList<>());
            Map<String, String> dateToBcxxMap = scheduleInfoList.stream().collect(Collectors.toMap(e->Util.null2String(e.get("bcrq")),e->Util.null2String(e.get("bcxx"))));

            List<String> restDateList = new ArrayList<>();
            if (removeNonWorkDayRange) {
                //排班结果中休息的日期
                List<String> restDateListFromSchedule = scheduleInfoList.stream()
                        .filter(f -> Util.null2String(f.get("sfxx")).equals(CheckBoxEnum.CHECKED.getKey()))
                        .map(e -> Util.null2String(e.get("bcrq")))
                        .collect(Collectors.toList());
                List<String> allDateListFromSchedule = scheduleInfoList.stream().map(e -> Util.null2String(e.get("bcrq"))).collect(Collectors.toList());
                //排班结果中未出现的日期
                List<String> nonSetDateListFromSchedule = leaveDateList.stream().filter(f -> !allDateListFromSchedule.contains(f)).collect(Collectors.toList());
                //企业日历中人员对应日期的休息日
                restDateList = restDayInfo.get(leaveEmpId);
                //筛选排班信息无法辨别的日期，依靠企业日历的日期信息
                restDateList = restDateList.stream().filter(nonSetDateListFromSchedule::contains).collect(Collectors.toList());
                restDateList.addAll(restDateListFromSchedule);
            }
            simpleLeaveDetailItem = new HashMap<>();

            String realStartDate = startDate;
            String realStartTime = startTime;
            String realEndDate = endDate;
            String realEndTime = endTime;
            if (halfDayRangeSelect.equals(CheckBoxEnum.CHECKED.getKey())) {
                //根据“上下午选择”字段来重新获取开始和结束时间点
                //获取开始、结束日期班次id
                String startDayBcId = restDateList.contains(startDate) ? "" : Util.null2String(dateToBcxxMap.get(startDate)).split("-")[0];
                String endDayBcId = restDateList.contains(endDate) ? "" : Util.null2String(dateToBcxxMap.get(endDate)).split("-")[0];
                if ("".equals(startDayBcId) || "".equals(endDayBcId)) {
                    continue;
                }
                //获取开始时间、结束时间
                String startFromBc = getTimePointFromBc(startDate, shiftInfoMap.get(startDayBcId), "start", startRange);
                String endFromBc = getTimePointFromBc(endDate, shiftInfoMap.get(endDayBcId),"end", endRange);
                if ("".equals(startFromBc) || "".equals(endFromBc)) {
                    continue;
                }
                realStartDate = startFromBc.split(" ")[0];
                realStartTime = startFromBc.split(" ")[1];
                realEndDate = endFromBc.split(" ")[0];
                realEndTime = endFromBc.split(" ")[1];
            }
            //组装初步的请假明细数据
            simpleLeaveDetailItem.put("qjr", leaveEmpId);
            simpleLeaveDetailItem.put("ksrq", realStartDate);
            simpleLeaveDetailItem.put("jsrq", realEndDate);
            simpleLeaveDetailItem.put("kssj", realStartTime);
            simpleLeaveDetailItem.put("jssj", realEndTime);
            for (String date : leaveDateList) {
                targetDateBcId = restDateList.contains(date) ? "" : Util.null2String(dateToBcxxMap.get(date)).split("-")[0];
                if (!"".equals(targetDateBcId)) {
                    Map<String, Object> shiftInfo = shiftInfoMap.get(targetDateBcId);
                    dateToRatedHours.put(date, Double.parseDouble(shiftInfo == null ? "8" : shiftInfo.get("edsc").toString()));

                    sql = "select id, bdlx, gsrq, kssj as dtkssj, jssj as dtjssj from uf_jcl_kq_bcxx_dt1 where mainid  = " + targetDateBcId;
                    bcDetailData = DbTools.getSqlToList(sql);
                    bcDetailData = bcDetailData.stream().filter(e -> countBdlxList.contains(Util.null2String(e.get("bdlx")))).collect(Collectors.toList());
                    //获取需要累计的班段时长区间和请假区间存在交集的分钟数
                    scMinutes = scMinutes + Utils.removeTime(realStartDate + " " + realStartTime, realEndDate + " " + realEndTime, bcDetailData, date);
                }
            }
            empIdToDateRatedHours.put(leaveEmpId, dateToRatedHours);
            //增加加班计划中的数据，加班计划明细中的数据作为“加班计划”班段类型数据的补充
            if (countBdlxList.contains(ClassSegmentTypeEnum.OVERTIME_PLAN.getKey())) {
                sql = "select b.jbry,b.ksrq,b.kssj,b.jblx,b.jsrq,b.jssj,b.jbsc,b.gsrq from uf_jcl_kq_jbjh a left join uf_jcl_kq_jbjh_dt1 b on a.id=b.mainid where b.jbry =? and b.ksrq>=? and b.jsrq<=? and (b.jbcx=0 or b.jbcx is null) and a.jlzt=1";
                List<Map<String,Object>> overtimePlanList = DbTools.getSqlToList(sql, leaveEmpId, DateUtil.beforeDay(realStartDate,1), DateUtil.AfterDay(realEndDate,1));
                int scMinutesInOvertimePlan = Utils.removeTimeWithOvertimePlan(realStartDate + " " + realStartTime, realEndDate + " " + realEndTime, overtimePlanList, dateToBcxxMap);
                scMinutes = scMinutes + scMinutesInOvertimePlan;
            }
            if (scMinutes > 0) {
                double qjscByAccount = Utils.getItemduration(hsl, hsdw, scMinutes, AccountingUnitEnum.MINUTES, 8.0);
                //组装初步的请假时长
                simpleLeaveDetailItem.put("qjsc", hsdw.equals(AccountingUnitEnum.MINUTES.getKey())
                        ? String.format("%.2f", qjscByAccount / 60.0)
                        : String.valueOf(qjscByAccount));
                simpleLeaveDetailList.add(simpleLeaveDetailItem);
            }
        }
        return simpleLeaveDetailList;
    }

    private String getTimePointFromBc(String leaveDate, Map<String, Object> shiftInfo, String pointType, String timeRange) {
        String sql = "select id, bdlx, gsrq, kssj as dtkssj, jssj as dtjssj, edts, edxss from uf_jcl_kq_bcxx_dt1 where mainid = " + shiftInfo.get("id")
                + " and bdlx = " + ClassSegmentTypeEnum.WORK_TIME.getKey() + " order by gsrq, kssj";
        List<Map<String, Object>> bdData = DbTools.getSqlToList(sql);
        if (bdData.size() == 0) {
            return "";
        }
        //根据半天规则，处理班段的上下半天数据
        String halfDayRuleValue = shiftInfo.get("btgz").toString();
        double ratedSc = Double.parseDouble(shiftInfo.get("edsc").toString());
        double totalSc = Double.parseDouble(shiftInfo.get("zgzsc").toString());
        String wholeBdStartTime = Utils.getkssjTime(bdData.get(0), leaveDate);
        String wholeBdEndTime = Utils.getjssjTime(bdData.get(bdData.size() - 1), leaveDate);
        String firstHalfEndTime = "";
        String secondHalfStartTime = "";
        //按班次设置　取固定时间　取总时长一半　取额定时长一半
        if (HalfDayRuleREnum.BY_CLASS_SET.getKey().equals(halfDayRuleValue)) {
            //按班次设置，根据班段明细中的额定天数来推断半天时间段，累计额定天数达到0.5天的时间点为上午结束的时间点，之后开始的工作时段的时间点为下午开始的时间点
            double countDays = 0;
            for (Map<String, Object> bdItem : bdData) {
                if (countDays == 0.50) {
                    secondHalfStartTime = Utils.getkssjTime(bdItem, leaveDate);
                    break;
                }
                double itemEdts = Double.parseDouble(bdItem.get("edts").toString());
                if (countDays + itemEdts > 0.50) {
                    double multiple = (0.50 - countDays) / itemEdts;
                    String itemStartTime = Utils.getkssjTime(bdItem, leaveDate);
                    String itemEndTime = Utils.getjssjTime(bdItem, leaveDate);
                    int minutes = (int) (DateUtil.getBetWeenMinutes(itemStartTime, itemEndTime) * multiple);
                    firstHalfEndTime = DateUtil.AfterMinutes(itemStartTime, minutes);
                    secondHalfStartTime = firstHalfEndTime;
                    break;
                } else if (countDays + itemEdts == 0.50) {
                    firstHalfEndTime = Utils.getjssjTime(bdItem, leaveDate);
                }
                countDays = countDays + itemEdts;
            }
        } else if (HalfDayRuleREnum.FIXED_DURATION.getKey().equals(halfDayRuleValue)) {
            //取固定时间，根据“分隔时间点”字段fgsjd来指定上午和下午分割的时间点
            String segmentTime = shiftInfo.get("fgsjd").toString();
            segmentTime = getSegmentTime(segmentTime, wholeBdStartTime, wholeBdEndTime);
            if (segmentTime.equals("")) {
                return "";
            }
            firstHalfEndTime = segmentTime;
            secondHalfStartTime = segmentTime;
        } else if (HalfDayRuleREnum.HALF_TOTAL_DURATION.getKey().equals(halfDayRuleValue)) {
            //取总时长一半，根据工作总时长的一半小时数，来确定半天小时数，累加班段明细的额定时长达到半天小时数的时间点为上午结束时间点
            double countHours = 0;
            for (Map<String, Object> bdItem : bdData) {
                if (countHours == totalSc / 2) {
                    secondHalfStartTime = Utils.getkssjTime(bdItem, leaveDate);
                    break;
                }
                double itemEdxss = Double.parseDouble(bdItem.get("edxss").toString());
                if (countHours + itemEdxss > totalSc / 2) {
                    String itemStartTime = Utils.getkssjTime(bdItem, leaveDate);
                    int diffMinutes = (int) ((totalSc / 2 - countHours) * 60);
                    firstHalfEndTime = DateUtil.AfterMinutes(itemStartTime, diffMinutes);
                    secondHalfStartTime = firstHalfEndTime;
                    break;
                } else if (countHours + itemEdxss == totalSc / 2) {
                    firstHalfEndTime = Utils.getjssjTime(bdItem, leaveDate);
                }
                countHours = countHours + itemEdxss;
            }
        } else if (HalfDayRuleREnum.HALF_RATED_DURATION.getKey().equals(halfDayRuleValue)) {
            //取额定时长一半，根据主表额定时长的一半小时数，来确定半天小时数，累加班段明细的额定时长达到半天小时数的时间点为上午结束时间点
            double countHours = 0;
            for (Map<String, Object> bdItem : bdData) {
                if (countHours == ratedSc / 2) {
                    secondHalfStartTime = Utils.getkssjTime(bdItem, leaveDate);
                    break;
                }
                double itemEdxss = Double.parseDouble(bdItem.get("edxss").toString());
                if (countHours + itemEdxss > ratedSc / 2) {
                    String itemStartTime = Utils.getkssjTime(bdItem, leaveDate);
                    int diffMinutes = (int) ((ratedSc / 2 - countHours) * 60);
                    firstHalfEndTime = DateUtil.AfterMinutes(itemStartTime, diffMinutes);
                    secondHalfStartTime = firstHalfEndTime;
                    break;
                } else if (countHours + itemEdxss == ratedSc / 2) {
                    firstHalfEndTime = Utils.getjssjTime(bdItem, leaveDate);
                }
                countHours = countHours + itemEdxss;
            }
        }

        if ("0".equals(timeRange)) {
            return "start".equals(pointType) ? wholeBdStartTime : firstHalfEndTime;
        } else if ("1".equals(timeRange)) {
            return "start".equals(pointType) ? secondHalfStartTime : wholeBdEndTime;
        } else {
            return "";
        }
    }

    private String getSegmentTime(String segmentTime, String wholeBdStartTime, String wholeBdEndTime) {
        String startDate = wholeBdStartTime.split(" ")[0];
        String endDate = wholeBdEndTime.split(" ")[0];
        String startPoint = wholeBdStartTime.split(" ")[1];
        String endPoint = wholeBdEndTime.split(" ")[1];
        if (startDate.equals(endDate)) {
            if (segmentTime.compareTo(startPoint) > 0 && segmentTime.compareTo(endPoint) < 0) {
                segmentTime = startDate + " " + segmentTime;
                return segmentTime;
            } else {
                return "";
            }
        }
        String waitJudgeSegTime = "";
        int waitCompareMinutes = 9999;
        for (int i = 0; i < 3; i++) {
            String matchSegTime = DateUtil.AfterDay(startDate,i) + " " + segmentTime;
            if (matchSegTime.compareTo(wholeBdStartTime) > 0 && matchSegTime.compareTo(wholeBdEndTime) < 0) {
                int diffMinutes = Math.abs(DateUtil.getBetWeenMinutes(wholeBdStartTime, matchSegTime) - DateUtil.getBetWeenMinutes(matchSegTime, wholeBdEndTime));
                waitCompareMinutes = Math.min(waitCompareMinutes, diffMinutes);
                waitJudgeSegTime = waitCompareMinutes == diffMinutes ? matchSegTime : waitJudgeSegTime;
            }
        }
        return waitJudgeSegTime;
    }

    /**
     * @param qjry 请假人员id
     * @param detailTableData 请假明细
     * @param errorMessage 错误信息
     * @param empName 请假人姓名
     * @return 校验该人员的考勤周期是否正常
     */
    private boolean kqCycleCheck(String qjry, List<Map<String, String>> detailTableData, List<String> errorMessage, String empName) {

        Map<String, Object> params = new HashMap<>();
        params.put("userId",qjry);
        params.put("submitDate",DateUtil.getCurrentDate());
        params.put("submitStr","ksrq");
        params.put("submitDataList",detailTableData);
        if (detailTableData == null || detailTableData.size() == 0){
            errorMessage.add(empName + "没有请假明细数据！");
            return false;
        }
        Map<String,Object> dataMap = makeUpClockInService.getKqCycleTimeIntervalCmd(params);
        List<Map<String,Object>> closeList = (List<Map<String,Object>>)dataMap.get("closeList");
        List<String> nocycleList = (List<String>)dataMap.get("nocycleList");
//        List<Map<String,Object>> dateList = (List<Map<String,Object>>)dataMap.get("dataList");

        boolean status = (boolean)dataMap.get("status");
        if (!status){
            errorMessage.add(empName + "没有考勤周期！");
            return false;
        }
        if (closeList.size() > 0 || nocycleList.size() > 0){
            String message = empName + "：";
            if (nocycleList.size() > 0){
                message = message +String.join(",",nocycleList)+"未找对对应的考勤周期；";
            }
            if (closeList.size() > 0){
                List<String> list = closeList.stream().map(e->e.get("rq").toString()).collect(Collectors.toList());
                message = message +String.join(",",list)+"对应的考勤周期的考勤周期已关账";
            }
            errorMessage.add(message);
            return false;
        }
        return true;
    }

//    /**
//     * @param editedLeaveInfo 触发接口前已编辑的请假明细信息
//     * @param checkAmountJqIdList 需要校验假期额度的假期类型id集合
//     * @param errorMessage 错误信息记录集合
//     * @param empId 请假人员id
//     * @param empName 请假人员姓名
//     * @param canUseCheckJqyeInfo 勾选的假期类型可使用的假期余额信息
//     * @param canUseFirstJqyeInfo 优先使用的假期类型可使用的假期余额信息
//     * @param editedUseJqed 触发接口前已编辑的假期额度使用信息
//     * @param editedUseJqlxWithEmp 触发接口前,本次提交的日期集合中已编辑的假期类型id_人员id条件下已使用的时长
//     * @param leaveDetailList 请假明细集合
//     * @param checkItemId 勾选的假期类型id
//     * @param firstItemId 优先使用的假期类型id
//     * @return 给请假明细中请假时长分配应扣的假期余额id及时长
//     */
//    private List<Map<String, String>> matchHolidayBalance(Map<String, String> editedLeaveInfo, List<String> checkAmountJqIdList, List<String> errorMessage, String empId, String empName,
//                                                          List<Map<String, Object>> canUseCheckJqyeInfo, List<Map<String, Object>> canUseFirstJqyeInfo, Map<String, Double> editedUseJqed,
//                                                          Map<String, Double> editedUseJqlxWithEmp,List<Map<String, String>> leaveDetailList, String checkItemId, String firstItemId,
//                                                          String checkItemName, String firstItemName) {
//        List<Map<String, String>> result = new ArrayList<>();
//        //记录分配假期余额的结果
//        boolean matchResultSign = true;
//        //此次遍历请假总时长
//        double totalLeaveDuration = 0;
//        for (Map<String, String> detailData : leaveDetailList) {
//            String qtj = Util.null2String(detailData.get("qtj"));
//            String btj = Util.null2String(detailData.get("btj"));
//            //请假时间
//            double leaveDuration;
//            if (qtj.equals(CheckBoxEnum.CHECKED.getKey())) {
//                //全天默认8小时
//                leaveDuration = 8;
//            } else if (btj.equals(CheckBoxEnum.CHECKED.getKey())) {
//                //半天默认4小时
//                leaveDuration = 4;
//            } else{
//                leaveDuration = Double.parseDouble(Util.null2String(detailData.get("qjsc")));
//            }
//            totalLeaveDuration = totalLeaveDuration + leaveDuration;
//        }
//        //已编辑明细中已使用时长
//        Double checkItemEditedUseSc = editedUseJqlxWithEmp.getOrDefault(checkItemId + "_" + empId, (double) 0);
//        Double firstItemEditedUseSc = editedUseJqlxWithEmp.getOrDefault(firstItemId + "_" + empId, (double) 0);
////        totalLeaveDuration = Utils.subtract(totalLeaveDuration, checkItemEditedUseSc == null ? 0 : checkItemEditedUseSc);
////        totalLeaveDuration = Utils.subtract(totalLeaveDuration, firstItemEditedUseSc == null ? 0 : firstItemEditedUseSc);
////        //如果总时长为0，则退出
//        if (Utils.add(checkItemEditedUseSc, firstItemEditedUseSc) > 0) {
//            String message = empName + "在该请假类型的本次请假日期范围内，已存在开始日期相同的请假明细，请重新选择日期范围！";
//            errorMessage.add(message);
//            return result;
//        }
//        Map<String, Double> leaveDurationMap = new HashMap<>();
//        leaveDurationMap.put("totalLeaveDuration", totalLeaveDuration);
//        //默认存在优先使用项目时长
//        boolean firstItemHaveDuration = true;
//        //优先使用的假期类型项目是否不需要校验假期余额的额度
//        boolean firstItemNonCheck = firstItemId != null && !checkAmountJqIdList.contains(firstItemId);
//        //勾选的假期类型项目是否不需要校验假期余额的额度
//        boolean checkItemNonCheck = !checkAmountJqIdList.contains(checkItemId);
//        //遍历关联请假余额id和假期类型id
//        for (Map<String, String> detailData : leaveDetailList) {
//            String ksrq = Util.null2String(detailData.get("ksrq"));
//            String qtj = Util.null2String(detailData.get("qtj"));
//            String btj = Util.null2String(detailData.get("btj"));
//            detailData.put("qjrName", empName);
//            //请假时间
//            double leaveDuration;
//            if (qtj.equals(CheckBoxEnum.CHECKED.getKey())) {
//                //全天默认8小时
//                leaveDuration = 8;
//            } else if (btj.equals(CheckBoxEnum.CHECKED.getKey())) {
//                //半天默认4小时
//                leaveDuration = 4;
//            } else{
//                leaveDuration = Double.parseDouble(Util.null2String(detailData.get("qjsc")));
//            }
//            log.info("leaveDuration : [{}]", leaveDuration);
//            //优先使用的的请假项目是否已被编辑
//            double firstItemEditedDetailSc = editedLeaveInfo.get(empId + "_" + firstItemId + "_" + ksrq) == null ? 0 : Double.parseDouble(editedLeaveInfo.get(empId + "_" + firstItemId + "_" + ksrq));
//            boolean firstItemEditedDetail = editedLeaveInfo.get(empId + "_" + firstItemId + "_" + ksrq) != null;
//            //勾选的请假项目是否已被编辑，此处判定包含了优先使用的请假项目编辑记录中的时长大于等于提交请假需求者的时长
//            double checkItemEditedDetailSc = editedLeaveInfo.get(empId + "_" + checkItemId + "_" + ksrq) == null ? 0 : Double.parseDouble(editedLeaveInfo.get(empId + "_" + checkItemId + "_" + ksrq));
//            boolean checkItemEditedDetail = editedLeaveInfo.get(empId + "_" + checkItemId + "_" + ksrq) != null || (firstItemEditedDetail && firstItemEditedDetailSc - leaveDuration >= 0);
//
//            leaveDuration = Utils.subtract(leaveDuration, firstItemEditedDetailSc);
//            leaveDuration = Utils.subtract(leaveDuration, checkItemEditedDetailSc);
//            if (leaveDuration <= 0) {
//                continue;
//            }
//            //存在优先使用假期类型时,且该假期类型不需要进行额度校验
//            if (firstItemNonCheck && !firstItemEditedDetail) {
//                detailData.put("qjlx", firstItemId);
//                detailData.put("qjlxName", firstItemName);
//                detailData.put("qjsc", String.valueOf(leaveDuration));
//                result.add(detailData);
//                continue;
//            }
//            //不存在优先使用假期类型时，勾选的假期类型不需要进行额度校验
//            if (firstItemId == null && checkItemNonCheck && !checkItemEditedDetail) {
//                detailData.put("qjlx", checkItemId);
//                detailData.put("qjlxName", checkItemName);
//                detailData.put("qjsc", String.valueOf(leaveDuration));
//                result.add(detailData);
//                continue;
//            }
//
//            leaveDurationMap.put("leaveDuration", leaveDuration);
//            //存在优先使用假期类型时,但是需要进行额度校验
//            if (canUseFirstJqyeInfo != null && canUseFirstJqyeInfo.size() > 0 && firstItemHaveDuration && !firstItemEditedDetail) {
//                dealLeaveDetailWithJqyeInfo(detailData, canUseFirstJqyeInfo, editedUseJqed, leaveDurationMap, ksrq, firstItemId, firstItemName, result);
//                if (leaveDurationMap.get("leaveDuration") == 0) {
//                    //如果同假期类型出现多假期余额拼接配置一条请假明细时，请假明细的时长需要重新设置下
//                    //请假明细初始要求时长已配置完成则赋值为初始请假时长，否则为初始时长-剩余未配置时长
//                    if (Util.null2String(detailData.get("jqye")).contains(",")) {
//                        detailData.put("qjsc", String.valueOf(leaveDuration));
//                    }
//                    continue;
//                } else {
//                    firstItemHaveDuration = false;
//                    if (Util.null2String(detailData.get("jqye")).contains(",")) {
//                        detailData.put("qjsc", String.valueOf(leaveDuration - leaveDurationMap.get("leaveDuration")));
//                    }
//                }
//            }
//            if (!"".equals(Util.null2String(detailData.get("qjlx")))) {
//                result.add(new HashMap<>(detailData));
//                detailData.remove("qjlx");
//                detailData.remove("qjlxName");
//                detailData.remove("qjsc");
//                detailData.remove("jqye");
//            }
//            //不存在优先使用假期类型，或者优先使用假期类型关联的假期余额用完时
//            //勾选的假期类型不需要进行额度校验
//            if (checkItemNonCheck && !checkItemEditedDetail) {
//                detailData.put("qjlx", checkItemId);
//                detailData.put("qjlxName", checkItemName);
//                detailData.put("qjsc", String.valueOf(leaveDurationMap.get("leaveDuration")));
//                leaveDurationMap.put("totalLeaveDuration", Utils.subtract(leaveDurationMap.get("totalLeaveDuration"), leaveDurationMap.get("leaveDuration")));
//                result.add(detailData);
//                continue;
//            }
//            //勾选的假期类型需要进行额度校验
//            boolean checkItemDealSign = true;
//            if (!checkItemEditedDetail) {
//                checkItemDealSign = dealLeaveDetailWithJqyeInfo(detailData, canUseCheckJqyeInfo, editedUseJqed, leaveDurationMap, ksrq, checkItemId, checkItemName, result);
//            }
//            if (leaveDurationMap.get("leaveDuration") > 0) {
//                String message = empName;
//                if (checkItemEditedDetail) {
//                    //人员id+日期+假期类型三种条件约束唯一性
//                    message = message + "_" + ksrq + "已存在一笔该请假类型的请假明细！";
//                } else if(!checkItemDealSign) {
//                    message = message + "_" + ksrq  + "不存在可使用的假期余额，请检查假期余额使用次数和最小使用时长设置！";
//                } else {
//                    //假期余额不足
//                    message = message + "_" + ksrq  + "假期余额不足！";
//                }
//                errorMessage.add(message);
//                matchResultSign = false;
//            }
//        }
//        return matchResultSign ? result : new ArrayList<>();
//    }


    /**
     * @param editedLeaveInfo 触发接口前已编辑的请假明细信息
     * @param checkAmountJqIdList 需要校验假期额度的假期类型id集合
     * @param errorMessage 错误信息记录集合
     * @param empId 请假人员id
     * @param empName 请假人员姓名
     * @param canUseJqyeInfo 可使用的假期余额信息
     * @param editedUseJqed 触发接口前已编辑的假期额度使用信息
     * @param editedUseJqlxWithEmp 触发接口前,本次提交的日期集合中已编辑的假期类型id_人员id条件下已使用的时长
     * @param leaveDetailList 请假明细集合
     * @return 给请假明细中请假时长分配应扣的假期余额id及时长
     */
    private List<Map<String, String>> matchHolidayWithPriority(Map<String, String> editedLeaveInfo, List<String> checkAmountJqIdList, List<String> errorMessage, String empId, String empName,
                                                               Map<String, List<Map<String, Object>>> canUseJqyeInfo, List<Map<String, Object>> holidayPriorityItemList, Map<String, Double> editedUseJqed,
                                                          Map<String, Double> editedUseJqlxWithEmp, List<Map<String, String>> leaveDetailList, List<Map<String, String>> editedLeaveInfoList,
                                                               Map<String, String> jbIdToHsdw, Map<String, Double> dateToRatedHours) {
        List<Map<String, String>> result = new ArrayList<>();
        //记录分配假期余额的结果
        boolean matchResultSign = true;
        //此次遍历请假总时长
        double totalLeaveHours = 0;
        double totalLeaveDays = 0;
        for (Map<String, String> detailData : leaveDetailList) {
            String qtj = Util.null2String(detailData.get("qtj"));
            String btj = Util.null2String(detailData.get("btj"));
            //请假时间
            double leaveHours;
            if (qtj.equals(CheckBoxEnum.CHECKED.getKey())) {
                //全天默认8小时
                leaveHours = Double.parseDouble(detailData.getOrDefault("qjscHours", "8"));
                totalLeaveDays = totalLeaveDays + 1;
            } else if (btj.equals(CheckBoxEnum.CHECKED.getKey())) {
                //半天默认4小时
                leaveHours = Double.parseDouble(detailData.getOrDefault("qjscHours", "4"));
                totalLeaveDays = totalLeaveDays + 0.5;
            } else {
                leaveHours = Double.parseDouble(Util.null2String(detailData.get("qjsc")));
            }
            totalLeaveHours = totalLeaveHours + leaveHours;
        }
        Map<String, Double> leaveDurationMap = new HashMap<>();
        leaveDurationMap.put("totalLeaveHours", totalLeaveHours);
        leaveDurationMap.put("totalLeaveDays", totalLeaveDays);
        //遍历关联请假余额id和假期类型id
        for (Map<String, String> detailData : leaveDetailList) {
            String ksrq = Util.null2String(detailData.get("ksrq"));
            String qtj = Util.null2String(detailData.get("qtj"));
            String btj = Util.null2String(detailData.get("btj"));
            detailData.put("qjrName", empName);
            //请假时间
            double leaveHours;
            double leaveDays = 0;
            if (qtj.equals(CheckBoxEnum.CHECKED.getKey())) {
                //全天默认8小时
                leaveHours = Double.parseDouble(detailData.getOrDefault("qjscHours", "8"));
                leaveDays = 1;
            } else if (btj.equals(CheckBoxEnum.CHECKED.getKey())) {
                //半天默认4小时
                leaveHours = Double.parseDouble(detailData.getOrDefault("qjscHours", "4"));
                leaveDays = 0.5;
            } else{
                leaveHours = Double.parseDouble(Util.null2String(detailData.get("qjsc")));
            }
            //单日假期“全局唯一性”约束校验
            boolean dailyCheckSign = checkDailyLeaveLicense(empId, empName, detailData, editedLeaveInfoList, leaveHours, errorMessage, dateToRatedHours, jbIdToHsdw);
            if (!dailyCheckSign) {
                leaveDurationMap.put("totalLeaveHours", Utils.subtract(leaveDurationMap.get("totalLeaveHours"), leaveHours));
                leaveDurationMap.put("totalLeaveDays", Utils.subtract(leaveDurationMap.get("totalLeaveDays"), leaveDays));
                continue;
            }

            log.info("leaveHours : [{}]", leaveHours);
            log.info("leaveDays : [{}]", leaveDays);
            leaveDurationMap.put("leaveHours", leaveHours);
            leaveDurationMap.put("leaveDays", leaveDays);
            //20241021逻辑变更，按照假期项目级联优先级，由后往前依次分配请假时长
            List<Map<String, Object>> jqyeList;
            String jqlxId;
            String jqlxName;
            String hsdw;
            boolean jqlxEdited = false;
            boolean dealSign = true;
            for (Map<String, Object> jqlxItem : holidayPriorityItemList) {
                jqlxId = Util.null2String(jqlxItem.get("id"));
                jqlxName = Util.null2String(jqlxItem.get("mc"));
                hsdw = Util.null2String(jqlxItem.get("hsdw"));
                jqyeList = canUseJqyeInfo.get(jqlxId);
                Double jqlxEditedUseSc = editedUseJqlxWithEmp.getOrDefault(jqlxId + "_" + empId, (double) 0);
                //使用的的请假项目是否已被编辑
                jqlxEdited = editedLeaveInfo.get(empId + "_" + jqlxId + "_" + ksrq) != null;
                //本次请假日期范围内，已存在该请假类型开始日期相同的请假明细，则不使用该请假类型
                if (jqlxEditedUseSc > 0 || jqlxEdited) {
                    jqlxEdited = true;
                    continue;
                }
                leaveHours = leaveDurationMap.get("leaveHours");
                leaveDays = leaveDurationMap.get("leaveDays");
                //使用的假期项目需要额度校验的话
                if (checkAmountJqIdList.contains(jqlxId)) {

                    dealSign = dealLeaveDetailWithJqyeInfo(detailData, jqyeList, editedUseJqed, leaveDurationMap, ksrq, jqlxId, jqlxName, result, dateToRatedHours);
                    if (!dealSign) {
                        continue;
                    }
                    double leaveDuration;
                    double initialLeaveDuration;
                    if (hsdw.equals(AccountingUnitEnum.DAY.getKey())) {
                        leaveDuration = leaveDurationMap.get("leaveDays");
                        initialLeaveDuration = leaveDays;
                    } else {
                        leaveDuration = leaveDurationMap.get("leaveHours");
                        initialLeaveDuration = leaveHours;
                    }
                    if (leaveDuration == 0) {
                        //如果同假期类型出现多假期余额拼接配置一条请假明细时，请假明细的时长需要重新设置下
                        //请假明细初始要求时长已配置完成则赋值为初始请假时长，否则为初始时长-剩余未配置时长
                        if (Util.null2String(detailData.get("jqye")).contains(",")) {
                            detailData.put("qjsc", String.valueOf(initialLeaveDuration));
                        }
                        break;
                    } else {
                        if (Util.null2String(detailData.get("jqye")).contains(",")) {
                            detailData.put("qjsc", String.valueOf(initialLeaveDuration - leaveDuration));
                        }
                    }
                } else {
                    detailData.put("qjlx", jqlxId);
                    detailData.put("qjlxName", jqlxName);
                    detailData.put("qjsc", hsdw.equals(AccountingUnitEnum.DAY.getKey()) ? String.valueOf(leaveDays) : String.valueOf(leaveHours));
                    result.add(detailData);
                    leaveDurationMap.put("totalLeaveDays", Utils.subtract(leaveDurationMap.get("totalLeaveDays"), leaveDurationMap.get("leaveDays")));
                    leaveDurationMap.put("leaveDays", (double) 0);
                    leaveDurationMap.put("totalLeaveHours", Utils.subtract(leaveDurationMap.get("totalLeaveHours"), leaveDurationMap.get("leaveHours")));
                    leaveDurationMap.put("leaveHours", (double) 0);
                    break;
                }
                if (!"".equals(Util.null2String(detailData.get("qjlx")))) {
                    result.add(new HashMap<>(detailData));
                    detailData.remove("qjlx");
                    detailData.remove("qjlxName");
                    detailData.remove("qjsc");
                    detailData.remove("jqye");
                }
            }

            if (leaveDurationMap.get("leaveHours") > 0) {
                String message = empName;
                if (jqlxEdited) {
                    //人员id+日期+假期类型三种条件约束唯一性
                    message = message + "_" + ksrq + "本流程中已存在一笔该请假类型的请假明细！";
                } else if(!dealSign) {
                    message = message + "_" + ksrq  + "不存在可使用的假期余额，请检查假期余额使用次数和最小使用时长设置！";
                } else {
                    //假期余额不足
                    message = message + "_" + ksrq  + "假期余额不足！";
                }
                errorMessage.add(message);
                matchResultSign = false;
            }
        }
        return matchResultSign ? result : new ArrayList<>();
    }


    /**
     * @param editedLeaveInfo 触发接口前已编辑的请假明细信息
     * @param checkAmountJqIdList 需要校验假期额度的假期类型id集合
     * @param errorMessage 错误信息记录集合
     * @param empId 请假人员id
     * @param empName 请假人员姓名
     * @param canUseJqyeList 可使用的假期余额列表，且已经按照“延期失效时间”+“级联优先值”排序后的列表
     * @param editedUseJqed 触发接口前已编辑的假期额度使用信息
     * @param editedUseJqlxWithEmp 触发接口前,本次提交的日期集合中已编辑的假期类型id_人员id条件下已使用的时长
     * @param leaveDetailList 请假明细集合
     * @return 给请假明细中请假时长分配应扣的假期余额id及时长
     */
    private List<Map<String, String>> matchHolidayWithDatePriority(Map<String, String> editedLeaveInfo, List<String> checkAmountJqIdList, List<String> errorMessage, String empId, String empName,
                                                               List<Map<String, Object>> canUseJqyeList, List<Map<String, Object>> holidayPriorityItemList, Map<String, Double> editedUseJqed,
                                                               Map<String, Double> editedUseJqlxWithEmp, List<Map<String, String>> leaveDetailList, List<Map<String, String>> editedLeaveInfoList,
                                                                   Map<String, String> jbIdToHsdw, Map<String, Double> dateToRatedHours) {
        List<Map<String, String>> result = new ArrayList<>();
        //记录分配假期余额的结果
        boolean matchResultSign = true;
        //此次遍历请假总时长
        double totalLeaveHours = 0;
        double totalLeaveDays = 0;
        for (Map<String, String> detailData : leaveDetailList) {
            String qtj = Util.null2String(detailData.get("qtj"));
            String btj = Util.null2String(detailData.get("btj"));
            //请假时间
            double leaveHours;
            if (qtj.equals(CheckBoxEnum.CHECKED.getKey())) {
                //全天默认8小时
                leaveHours = Double.parseDouble(detailData.getOrDefault("qjscHours", "8"));
                totalLeaveDays = totalLeaveDays + 1;
            } else if (btj.equals(CheckBoxEnum.CHECKED.getKey())) {
                //半天默认4小时
                leaveHours = Double.parseDouble(detailData.getOrDefault("qjscHours", "4"));
                totalLeaveDays = totalLeaveDays + 0.5;
            } else {
                leaveHours = Double.parseDouble(Util.null2String(detailData.get("qjsc")));
            }
            totalLeaveHours = totalLeaveHours + leaveHours;
        }
        Map<String, Double> leaveDurationMap = new HashMap<>();
        leaveDurationMap.put("totalLeaveHours", totalLeaveHours);
        leaveDurationMap.put("totalLeaveDays", totalLeaveDays);
        //遍历关联请假余额id和假期类型id
        for (Map<String, String> detailData : leaveDetailList) {
            String ksrq = Util.null2String(detailData.get("ksrq"));
            String qtj = Util.null2String(detailData.get("qtj"));
            String btj = Util.null2String(detailData.get("btj"));
            detailData.put("qjrName", empName);
            //请假时间
            double leaveHours;
            double leaveDays = 0;
            if (qtj.equals(CheckBoxEnum.CHECKED.getKey())) {
                //全天默认8小时
                leaveHours = Double.parseDouble(detailData.getOrDefault("qjscHours", "8"));
                leaveDays = 1;
            } else if (btj.equals(CheckBoxEnum.CHECKED.getKey())) {
                //半天默认4小时
                leaveHours = Double.parseDouble(detailData.getOrDefault("qjscHours", "4"));
                leaveDays = 0.5;
            } else{
                leaveHours = Double.parseDouble(Util.null2String(detailData.get("qjsc")));
            }
            //单日假期“全局唯一性”约束校验
            boolean dailyCheckSign = checkDailyLeaveLicense(empId, empName, detailData, editedLeaveInfoList, leaveHours, errorMessage, dateToRatedHours, jbIdToHsdw);
            if (!dailyCheckSign) {

                leaveDurationMap.put("totalLeaveHours", Utils.subtract(leaveDurationMap.get("totalLeaveHours"), leaveHours));
                leaveDurationMap.put("totalLeaveDays", Utils.subtract(leaveDurationMap.get("totalLeaveDays"), leaveDays));
                continue;
            }

            log.info("leaveHours : [{}]", leaveHours);
            log.info("leaveDays : [{}]", leaveDays);
            leaveDurationMap.put("leaveHours", leaveHours);
            leaveDurationMap.put("leaveDays", leaveDays);
            //20241107逻辑变更，按照假期余额延长失效日期+假期项目级联优先级，（失效日期越早+优先级越高）依次分配请假时长
            Map<String, String> jqlxIdToNameInfo = holidayPriorityItemList.stream().collect(Collectors.toMap(e->Util.null2String(e.get("id")),e->Util.null2String(e.get("mc"))));
            dealLeaveDetailWithAllJqye(detailData, canUseJqyeList, editedUseJqed, leaveDurationMap, ksrq, result, jqlxIdToNameInfo, editedLeaveInfo, editedUseJqlxWithEmp, empId, dateToRatedHours);
            //额度假匹配结束，如果还有请假时长未分配，筛选出假期中非额度假的假期类型来分配请假时长
            leaveHours = leaveDurationMap.get("leaveHours");
            leaveDays = leaveDurationMap.get("leaveDays");
            if (leaveHours > 0) {
                holidayPriorityItemList = holidayPriorityItemList.stream().filter(f -> !checkAmountJqIdList.contains(f.get("id").toString())).collect(Collectors.toList());
                if (holidayPriorityItemList.size() == 0) {
                    errorMessage.add(empName + "_" + ksrq  + "假期余额不足！");
                    matchResultSign = false;
                }
                boolean jqlxEdited = false;
                String hsdw;
                for (Map<String, Object> holidayItem : holidayPriorityItemList) {
                    Double jqlxEditedUseSc = editedUseJqlxWithEmp.getOrDefault(holidayItem.get("id") + "_" + empId, (double) 0);
                    //使用的的请假项目是否已被编辑
                    jqlxEdited = editedLeaveInfo.get(empId + "_" + holidayItem.get("id") + "_" + ksrq) != null;
                    //本次请假日期范围内，已存在该请假类型开始日期相同的请假明细，则不使用该请假类型
                    if (jqlxEditedUseSc > 0 || jqlxEdited) {
                        jqlxEdited = true;
                        continue;
                    }
                    hsdw = Util.null2String(holidayItem.get("hsdw"));
                    detailData.put("qjlx", holidayPriorityItemList.get(0).get("id").toString());
                    detailData.put("qjlxName", jqlxIdToNameInfo.get(holidayPriorityItemList.get(0).get("id").toString()));
                    detailData.put("qjsc", hsdw.equals(AccountingUnitEnum.DAY.getKey()) ? String.valueOf(leaveDays) : String.valueOf(leaveHours));
                    detailData.put("jqye", "");
                    result.add(detailData);
                    leaveHours = 0;
                    break;
                }
                if (leaveHours > 0) {
                    String message = empName;
                    if (jqlxEdited) {
                        //人员id+日期+假期类型三种条件约束唯一性
                        message = message + "_" + ksrq + "本流程中已存在一笔该请假类型的请假明细！";
                    } else {
                        //假期余额不足
                        message = message + "_" + ksrq + "假期余额不足！";
                    }
                    errorMessage.add(message);
                    matchResultSign = false;
                }
            }

        }
        return matchResultSign ? result : new ArrayList<>();
    }


    private boolean checkDailyLeaveLicense(String empId, String empName, Map<String, String> detailData, List<Map<String, String>> editedLeaveInfoList,
                                        double leaveHours, List<String> errorMessage, Map<String, Double> dateToRatedHours, Map<String, String> jbIdToHsdw) {
        //判断当前请假信息属于哪一种请假方式（全天、半天、指定时长、指定区间）
        String leaveMode = "";
        if (!"".equals(Util.null2String(detailData.get("kssj")))) {
            leaveMode = AskAndEvctionWayEnum.TIME_INTERVAL.getKey();
        } else if ("1".equals(Util.null2String(detailData.get("qtj")))) {
            leaveMode = AskAndEvctionWayEnum.ALLDAY.getKey();
        } else if ("1".equals(Util.null2String(detailData.get("btj")))) {
            leaveMode = AskAndEvctionWayEnum.HALFDAY.getKey();
        } else {
            leaveMode = AskAndEvctionWayEnum.HOUR.getKey();
        }
        String ksrq = Util.null2String(detailData.get("ksrq"));
        String jsrq = Util.null2String(detailData.get("jsrq"));
        String kssj = Util.null2String(detailData.get("kssj"));
        String jssj = Util.null2String(detailData.get("jssj"));
        //查询数据库中当天存在的请假记录
        List<Map<String, Object>> existLeaveInfo;
        String sql = "select b.* from uf_jcl_kq_qjjl a left join uf_jcl_kq_qjjl_dt1 b on b.mainid = a.id  where a.jlzt != 2 and b.cxqj = 0 and b.qjry = " + empId;
        if (leaveMode.equals(AskAndEvctionWayEnum.TIME_INTERVAL.getKey())) {
            sql = sql + " and ((b.ksrq >= ? and b.ksrq <= ?) or (b.jsrq >= ? and b.jsrq <= ?) or (b.ksrq <= ? and b.jsrq >= ?))";
            existLeaveInfo = DbTools.getSqlToList(sql, ksrq, jsrq, ksrq, jsrq, ksrq, jsrq);
        } else {
            sql = sql + " and b.ksrq <= ? and b.jsrq >= ?";
            existLeaveInfo = DbTools.getSqlToList(sql, ksrq, ksrq);
        }
        //分析数据库中已存在请假数据
        String message = empName;
        double existLeaveSc = 0;
        if (existLeaveInfo.size() > 0) {
            List<Map<String, Object>> timeIntervalLeaveList = new ArrayList<>();
            List<Map<String, Object>> allDayLeaveList = new ArrayList<>();
            List<Map<String, Object>> otherLeaveList = new ArrayList<>();
            for (Map<String, Object> leaveItem : existLeaveInfo) {
                if (!"".equals(Util.null2String(leaveItem.get("kssj")))) {
                    timeIntervalLeaveList.add(leaveItem);
                } else if ("1".equals(Util.null2String(leaveItem.get("qtj")))) {
                    allDayLeaveList.add(leaveItem);
                } else {
                    otherLeaveList.add(leaveItem);
                }
            }
            if (allDayLeaveList.size() > 0) {
                //该日期已存在全天假时
                message = message + "_" + ksrq  + "已存在全天假, 不允许再进行任何请假申请！";
                errorMessage.add(message);
                return false;
            } else if (leaveMode.equals(AskAndEvctionWayEnum.ALLDAY.getKey())) {
                //该日期即将生成全天假
                //首先判断，是否已存在区间假信息，且区间假区间与目标日期有重叠时间段，然后判断其他请假方式是由已经存在记录
                List<Map<String, Object>> overlappingList = timeIntervalLeaveList.stream().filter(f ->
                                DateUtil.isOverlapping(ksrq + " 00:00", ksrq + " 23:59", f.get("ksrq") + " " + f.get("kssj"), f.get("jsrq") + " " + f.get("jssj")))
                        .collect(Collectors.toList());
                if (overlappingList.size() > 0 || otherLeaveList.size() > 0) {
                    message = message + "_" + ksrq  + "已存在请假记录, 不允许再进行全天假方式的请假申请！";
                    errorMessage.add(message);
                    return false;
                }
            } else if (leaveMode.equals(AskAndEvctionWayEnum.TIME_INTERVAL.getKey())) {
                //该日期即将生成指定时间区间假
                //首先判断，是否已存在区间假之外的信息，然后判断是否存在区间假且存在交集
                List<Map<String, Object>> overlappingList = timeIntervalLeaveList.stream().filter(f ->
                        DateUtil.isOverlapping(ksrq + " " + kssj, jsrq + " " + jssj, f.get("ksrq") + " " + f.get("kssj"), f.get("jsrq") + " " + f.get("jssj")))
                        .collect(Collectors.toList());
                if (overlappingList.size() > 0 || otherLeaveList.size() > 0) {
                    message = message + "在" + ksrq + "开始的请假区间内已存在请假记录的区间重叠, 不允许再进行请假申请！";
                    errorMessage.add(message);
                    return false;
                }
            } else if (leaveMode.equals(AskAndEvctionWayEnum.HALFDAY.getKey()) || leaveMode.equals(AskAndEvctionWayEnum.HOUR.getKey())) {
                //该日期即将生成指定时间长度假或者半天假
                //首先判断，是否已存在区间假信息，且区间假区间与目标日期有重叠时间段
                List<Map<String, Object>> overlappingList = timeIntervalLeaveList.stream().filter(f ->
                        DateUtil.isOverlapping(ksrq + " 00:00", ksrq + " 23:59", f.get("ksrq") + " " + f.get("kssj"), f.get("jsrq") + " " + f.get("jssj")))
                        .collect(Collectors.toList());
                if (overlappingList.size() > 0) {
                    message = message + "_" + ksrq  + "已存在指定时间区间请假记录, 不允许再其他方式的请假申请！";
                    errorMessage.add(message);
                    return false;
                }
                //然后判断其他请假方式是由已经存在记录,结合该类请假记录的已有请假时长之和判断
                if (otherLeaveList.size() > 0) {
//                    existLeaveSc = otherLeaveList.stream().mapToDouble(e->Double.parseDouble(e.getOrDefault("qjsc", "0").toString())).sum();
                    existLeaveSc = otherLeaveList.stream().mapToDouble(e-> {
                        if (jbIdToHsdw.get(e.get("jqlx").toString()).equals(AccountingUnitEnum.DAY.getKey())) {
                            return Double.parseDouble(e.getOrDefault("qjsc", "0").toString()) * dateToRatedHours.getOrDefault(ksrq, 8.0);
                        }
                        return Double.parseDouble(e.getOrDefault("qjsc", "0").toString());
                    }).sum();
                    if (existLeaveSc + leaveHours > dateToRatedHours.getOrDefault(ksrq, 8.0)) {
                        message = message + "_" + ksrq  + "已存在共计" + existLeaveSc
                                + "小时的请假记录, 新申请假期累计后已超过当天上班额定时长" + dateToRatedHours.getOrDefault(ksrq, 8.0) + "小时！";
                        errorMessage.add(message);
                        return false;
                    }
                }
            }
        }
        //分析当前流程已编辑请假明细中已存在请假数据
        if (editedLeaveInfoList != null && editedLeaveInfoList.size() > 0) {
            //筛选出时间范围内的
            if (leaveMode.equals(AskAndEvctionWayEnum.TIME_INTERVAL.getKey())) {
                editedLeaveInfoList = editedLeaveInfoList.stream()
                        .filter(f -> (Util.null2String(f.get("ksrq")).compareTo(ksrq) >= 0 && Util.null2String(f.get("ksrq")).compareTo(jsrq) <= 0)
                                || (Util.null2String(f.get("jsrq")).compareTo(ksrq) >= 0 && Util.null2String(f.get("jsrq")).compareTo(jsrq) <= 0)
                                || (Util.null2String(f.get("ksrq")).compareTo(ksrq) <= 0 && Util.null2String(f.get("jsrq")).compareTo(jsrq) >= 0))
                        .collect(Collectors.toList());
            } else {
                editedLeaveInfoList = editedLeaveInfoList.stream()
                        .filter(f -> Util.null2String(f.get("ksrq")).compareTo(ksrq) <= 0 && Util.null2String(f.get("jsrq")).compareTo(ksrq) >= 0)
                        .collect(Collectors.toList());
            }
            List<Map<String, String>> timeIntervalLeaveList = new ArrayList<>();
            List<Map<String, String>> allDayLeaveList = new ArrayList<>();
            List<Map<String, String>> otherLeaveList = new ArrayList<>();
            for (Map<String, String> leaveItem : editedLeaveInfoList) {
                if (!"".equals(Util.null2String(leaveItem.get("kssj")))) {
                    timeIntervalLeaveList.add(leaveItem);
                } else if ("1".equals(Util.null2String(leaveItem.get("qtj")))) {
                    allDayLeaveList.add(leaveItem);
                } else {
                    otherLeaveList.add(leaveItem);
                }
            }
            if (allDayLeaveList.size() > 0) {
                //该日期已存在全天假时
                message = message + "_" + ksrq  + "在当前流程中已编辑了全天假, 不允许再进行任何请假申请！";
                errorMessage.add(message);
                return false;
            } else if (leaveMode.equals(AskAndEvctionWayEnum.ALLDAY.getKey())) {
                //该日期即将生成全天假
                //首先判断，是否已存在区间假信息，且区间假区间与目标日期有重叠时间段，然后判断其他请假方式是由已经存在记录
                List<Map<String, String>> overlappingList = timeIntervalLeaveList.stream().filter(f ->
                        DateUtil.isOverlapping(ksrq + " 00:00", ksrq + " 23:59", f.get("ksrq") + " " + f.get("kssj"), f.get("jsrq") + " " + f.get("jssj")))
                        .collect(Collectors.toList());
                if (overlappingList.size() > 0 || otherLeaveList.size() > 0) {
                    message = message + "_" + ksrq  + "在当前流程中已生成了请假记录, 不允许再进行全天假方式的请假申请！";
                    errorMessage.add(message);
                    return false;
                }
            } else if (leaveMode.equals(AskAndEvctionWayEnum.TIME_INTERVAL.getKey())) {
                //该日期即将生成指定时间区间假
                //首先判断，是否已存在区间假之外的信息，然后判断是否存在区间假且存在交集
                List<Map<String, String>> overlappingList = timeIntervalLeaveList.stream().filter(f ->
                        DateUtil.isOverlapping(ksrq + " " + kssj, jsrq + " " + jssj, f.get("ksrq") + " " + f.get("kssj"), f.get("jsrq") + " " + f.get("jssj")))
                        .collect(Collectors.toList());
                if (overlappingList.size() > 0 || otherLeaveList.size() > 0) {
                    message = message + "在" + ksrq + "开始的请假区间内已存在请假记录的区间重叠, 不允许再进行请假申请！";
                    errorMessage.add(message);
                    return false;
                }
            } else if (leaveMode.equals(AskAndEvctionWayEnum.HALFDAY.getKey()) || leaveMode.equals(AskAndEvctionWayEnum.HOUR.getKey())) {
                //该日期即将生成指定时间长度假或者半天假
                //首先判断，是否已存在区间假信息，且区间假区间与目标日期有重叠时间段
                List<Map<String, String>> overlappingList = timeIntervalLeaveList.stream().filter(f ->
                        DateUtil.isOverlapping(ksrq + " 00:00", ksrq + " 23:59", f.get("ksrq") + " " + f.get("kssj"), f.get("jsrq") + " " + f.get("jssj")))
                        .collect(Collectors.toList());
                if (overlappingList.size() > 0) {
                    message = message + "_" + ksrq  + "在当前流程中已存在指定时间区间请假记录, 不允许再其他方式的请假申请！";
                    errorMessage.add(message);
                    return false;
                }
                //然后判断其他请假方式是由已经存在记录,结合该类请假记录的已有请假时长之和判断
                if (otherLeaveList.size() > 0) {
//                    existLeaveSc = existLeaveSc + otherLeaveList.stream().mapToDouble(e->Double.parseDouble(e.getOrDefault("qjsc", "0").toString())).sum();
                    existLeaveSc = existLeaveSc + otherLeaveList.stream().mapToDouble(e-> {
                        if (jbIdToHsdw.get(e.get("qjlx").toString()).equals(AccountingUnitEnum.DAY.getKey())) {
                            return Double.parseDouble(e.getOrDefault("qjsc", "0").toString()) * dateToRatedHours.getOrDefault(ksrq, 8.0);
                        }
                        return Double.parseDouble(e.getOrDefault("qjsc", "0").toString());
                    }).sum();
                    if (existLeaveSc + leaveHours > dateToRatedHours.getOrDefault(ksrq, 8.0)) {
                        message = message + "_" + ksrq  + "已存在当天请假记录和当前流程已编辑请假明细共计" + existLeaveSc
                                + "小时的请假记录, 新申请假期累计后已超过当天上班额定时长" + dateToRatedHours.getOrDefault(ksrq, 8.0) + "小时！";
                        errorMessage.add(message);
                        return false;
                    }
                }
            }
        }

        //判断已存在出差记录和待生成请假信息是否存在冲突
        //查询数据库中当天存在的出差记录
        List<Map<String, Object>> existCcInfo;
        sql = "select b.* from uf_jcl_kq_ccjl a left join uf_jcl_kq_ccjl_dt1 b on b.mainid = a.id  where a.jlzt != 2 and b.cxcc= 0 and b.ccr = " + empId;
        if (leaveMode.equals(AskAndEvctionWayEnum.TIME_INTERVAL.getKey())) {
            sql = sql + " and ((b.ksrq >= ? and b.ksrq <= ?) or (b.jsrq >= ? and b.jsrq <= ?) or (b.ksrq <= ? and b.jsrq >= ?))";
            existCcInfo = DbTools.getSqlToList(sql, ksrq, jsrq, ksrq, jsrq, ksrq, jsrq);
        } else {
            sql = sql + " and b.ksrq <= ? and b.jsrq >= ?";
            existCcInfo = DbTools.getSqlToList(sql, ksrq, ksrq);
        }
        //分析数据库中已存在出差数据
        double existCcSc = 0;
        if (existCcInfo.size() > 0) {
            List<Map<String, Object>> timeIntervalCcList = new ArrayList<>();
            List<Map<String, Object>> allDayCcList = new ArrayList<>();
            List<Map<String, Object>> otherCcList = new ArrayList<>();
            for (Map<String, Object> leaveItem : existCcInfo) {
                if (!"".equals(Util.null2String(leaveItem.get("kssj")))) {
                    timeIntervalCcList.add(leaveItem);
                } else if ("1".equals(Util.null2String(leaveItem.get("qtcc")))) {
                    allDayCcList.add(leaveItem);
                } else {
                    otherCcList.add(leaveItem);
                }
            }
            if (allDayCcList.size() > 0) {
                //该日期已存在全天出差时
                message = message + "_" + ksrq  + "已存在全天出差, 不允许再进行任何请假申请！";
                errorMessage.add(message);
                return false;
            } else if (leaveMode.equals(AskAndEvctionWayEnum.ALLDAY.getKey())) {
                //该日期即将生成全天出差
                //首先判断，是否已存在区间出差信息，且区间出差区间与目标日期有重叠时间段，然后判断其他出差方式是由已经存在记录
                List<Map<String, Object>> overlappingList = timeIntervalCcList.stream().filter(f ->
                        DateUtil.isOverlapping(ksrq + " 00:00", ksrq + " 23:59", f.get("ksrq") + " " + f.get("kssj"), f.get("jsrq") + " " + f.get("jssj")))
                        .collect(Collectors.toList());
                if (overlappingList.size() > 0 || otherCcList.size() > 0) {
                    message = message + "_" + ksrq  + "已存在出差记录, 不允许再进行全天请假方式的请假申请！";
                    errorMessage.add(message);
                    return false;
                }
            } else if (leaveMode.equals(AskAndEvctionWayEnum.TIME_INTERVAL.getKey())) {
                //该日期即将生成指定时间区间出差
                //首先判断，是否已存在区间出差之外的信息，然后判断是否存在区间出差且存在交集
                List<Map<String, Object>> overlappingList = timeIntervalCcList.stream().filter(f ->
                        DateUtil.isOverlapping(ksrq + " " + kssj, jsrq + " " + jssj, f.get("ksrq") + " " + f.get("kssj"), f.get("jsrq") + " " + f.get("jssj")))
                        .collect(Collectors.toList());
                if (overlappingList.size() > 0 || otherCcList.size() > 0) {
                    message = message + "在" + ksrq + "到" + jsrq + "的出差区间内已存在出差记录, 不允许再进行请假申请！";
                    errorMessage.add(message);
                    return false;
                }
            } else if (leaveMode.equals(AskAndEvctionWayEnum.HALFDAY.getKey()) || leaveMode.equals(AskAndEvctionWayEnum.HOUR.getKey())) {
                //该日期即将生成指定时间长度出差或者半天出差
                //首先判断，是否已存在区间出差信息，且区间出差区间与目标日期有重叠时间段
                List<Map<String, Object>> overlappingList = timeIntervalCcList.stream().filter(f ->
                        DateUtil.isOverlapping(ksrq + " 00:00", ksrq + " 23:59", f.get("ksrq") + " " + f.get("kssj"), f.get("jsrq") + " " + f.get("jssj")))
                        .collect(Collectors.toList());
                if (overlappingList.size() > 0) {
                    message = message + "_" + ksrq  + "已存在指定时间区间出差记录, 不允许再其他方式的请假申请！";
                    errorMessage.add(message);
                    return false;
                }
                //然后判断其他出差方式是由已经存在记录,结合该类出差记录的已有出差时长之和判断
                if (otherCcList.size() > 0) {

                    existCcSc = otherCcList.stream().mapToDouble(e-> {
                        String itemCcsc  = "".equals(Util.null2String(e.get("ccsc"))) ? "0" : e.get("ccsc").toString();
                        if (jbIdToHsdw.get(e.get("cclx").toString()).equals(AccountingUnitEnum.DAY.getKey())) {
                            return Double.parseDouble(itemCcsc) * dateToRatedHours.getOrDefault(ksrq, 8.0);
                        }
                        if ("".equals(Util.null2String(e.get("ccsc"))) && "1".equals(Util.null2String(e.get("btcc"))) ) {
                            return 0.5 * dateToRatedHours.getOrDefault(ksrq, 8.0);
                        }
                        return Double.parseDouble(itemCcsc);
                    }).sum();
                    if (existLeaveSc + existCcSc + leaveHours > dateToRatedHours.getOrDefault(ksrq, 8.0)) {
                        message = message + "_" + ksrq  + "已存在共计" + existCcSc
                                + "小时的出差记录、" + existLeaveSc
                                + "小时的请假记录，新申请假期累计后已超过当天上班额定时长" + dateToRatedHours.getOrDefault(ksrq, 8.0) + "小时！";
                        errorMessage.add(message);
                        return false;
                    }
                }
            }
        }
        
        return true;
    }

    private boolean dealLeaveDetailWithJqyeInfo(Map<String, String> detailData, List<Map<String, Object>> canUseJqyeInfo,
                                             Map<String, Double> editedUseJqed, Map<String, Double> leaveDurationMap,
                                             String ksrq, String itemId, String itemName, List<Map<String, String>> result, Map<String, Double> dateToRatedHours) {
        double totalLeaveHours = leaveDurationMap.get("totalLeaveHours");
        double totalLeaveDays = leaveDurationMap.get("totalLeaveDays");
        double leaveHours = leaveDurationMap.get("leaveHours");
        double leaveDays = leaveDurationMap.get("leaveDays");
        //假期类型相关的假期余额记录
        List<Map<String, Object>> holidayBalancefilterList = new ArrayList<>();
        for (Map<String, Object> map : canUseJqyeInfo){
            if (DateUtil.getTime(map.get("yqsxrq").toString()).compareTo(DateUtil.getTime(ksrq)) >= 0){
                holidayBalancefilterList.add(map);
            }
        }
        if (holidayBalancefilterList.size() == 0) {
            return false;
        }
        //使用的假期类型的核算单位
        String hsdw = Util.null2String(holidayBalancefilterList.get(0).get("hsdw"));
        //20241118需求变更，全天、半天时长根据班次信息获取，以下请假时长分配逻辑的执行前提必须遵守“核算单位为‘天’的假期只能申请全天假、半天假！”
        //所以，只有在全天假和半天假时，leaveDurationMap中才会有请假天数信息；其他请假方式则会请假天数和请假小时数都有
        double totalLeaveDuration;
        double leaveDuration;
        if (hsdw.equals(AccountingUnitEnum.DAY.getKey())) {
            totalLeaveDuration = totalLeaveDays;
            leaveDuration = leaveDays;
        } else {
            totalLeaveDuration = totalLeaveHours;
            leaveDuration = leaveHours;
        }
        for (Map<String, Object> holidayBalance : holidayBalancefilterList) {
            //1-如果已编辑的请假明细中已有该假期余额id的记录，则满足最小使用时长;如果result的请假明细中已有该假期余额id的记录，则满足最小使用时长
            boolean minLeaveDurationSign = editedUseJqed.get(holidayBalance.get("id").toString()) != null || result.stream().anyMatch(map -> itemId.equals(map.get("qjlx")));
            //2-如果编辑记录中不存在，则需要判断是否满足最小使用时长
            if (!minLeaveDurationSign) {
                String minLeaveDuration = Util.null2String(holidayBalance.get("minLeaveDuration"));
                minLeaveDurationSign = "".equals(minLeaveDuration) || totalLeaveDuration >= Double.parseDouble(minLeaveDuration);
            }
            //不满足最小使用时长，直接执行下一条
            if (!minLeaveDurationSign) {
                return false;
            }
            //额定未休时长
            double wxsc = "".equals(Util.null2String(holidayBalance.get("wxsc"))) ? 0 : Double.parseDouble(holidayBalance.get("wxsc").toString());
            if (leaveDuration > 0  && wxsc > 0) {
                detailData.put("qjlx", itemId);
                detailData.put("qjlxName", itemName);
                String jqyeId = Util.null2String(detailData.get("jqye"));
                //已分配的请假时长
                String matchedQjsc = Util.null2String(detailData.get("qjsc"));
                if (leaveDuration > wxsc) {
                    detailData.put("qjsc", String.valueOf(wxsc));
                    leaveDuration = Utils.subtract(leaveDuration, wxsc);
                    totalLeaveDuration = Utils.subtract(totalLeaveDuration, wxsc);
                    if ("".equals(jqyeId)) {
                        detailData.put("jqye", holidayBalance.get("id").toString() + "_" + wxsc);
                    } else {
                        //同一假期类型，当一条假期余额记录不足请假时长需求时，需要拼接上另一条假期余额记录的id
                        detailData.put("jqye", jqyeId + "," + holidayBalance.get("id").toString() + "_" + wxsc);
                        detailData.put("qjsc", String.valueOf(Utils.add(wxsc, Double.parseDouble(matchedQjsc))));
                    }
                    wxsc = 0;
                } else {
                    detailData.put("qjsc", String.valueOf(leaveDuration));
                    wxsc = Utils.subtract(wxsc, leaveDuration);
                    totalLeaveDuration = Utils.subtract(totalLeaveDuration, leaveDuration);
                    if ("".equals(jqyeId)) {
                        detailData.put("jqye", holidayBalance.get("id").toString() + "_" + leaveDuration);
                    } else {
                        //同一假期类型，当一条假期余额记录不足请假时长需求时，需要拼接上另一条假期余额记录的id
                        detailData.put("jqye", jqyeId + "," + holidayBalance.get("id").toString() + "_" + leaveDuration);
                        detailData.put("qjsc", String.valueOf(Utils.add(leaveDuration, Double.parseDouble(matchedQjsc))));
                    }
                    leaveDuration = 0;
                    result.add(detailData);
                }

                holidayBalance.put("wxsc", wxsc);
            }
        }

        if (hsdw.equals(AccountingUnitEnum.DAY.getKey())) {
            //假别核算单位为‘天’时，需要更新leaveDurationMap中的请假天数和请假小时数
            leaveDurationMap.put("totalLeaveDays", totalLeaveDuration);
            leaveDurationMap.put("leaveDays", leaveDuration);
            double ratedDayHours = dateToRatedHours.getOrDefault(detailData.get("ksrq"), 8.0);
            leaveDurationMap.put("totalLeaveHours", totalLeaveHours - (leaveDays - leaveDuration) * ratedDayHours);
            leaveDurationMap.put("leaveHours", leaveHours - (leaveDays - leaveDuration) * ratedDayHours);
        } else {
            //假别核算单位为‘小时’时，只需要更新leaveDurationMap中的请假小时数
            leaveDurationMap.put("totalLeaveHours", totalLeaveDuration);
            leaveDurationMap.put("leaveHours", leaveDuration);
        }

        return true;
    }


    private void dealLeaveDetailWithAllJqye(Map<String, String> detailData, List<Map<String, Object>> canUseJqyeList, Map<String, Double> editedUseJqed,
                                               Map<String, Double> leaveDurationMap, String ksrq, List<Map<String, String>> result, Map<String, String> jqlxIdToNameInfo,
                                               Map<String, String> editedLeaveInfo, Map<String, Double> editedUseJqlxWithEmp, String empId, Map<String, Double> dateToRatedHours) {

        double totalLeaveHours = leaveDurationMap.get("totalLeaveHours");
        double totalLeaveDays = leaveDurationMap.get("totalLeaveDays");
        double leaveHours = leaveDurationMap.get("leaveHours");
        double leaveDays = leaveDurationMap.get("leaveDays");
        List<Map<String, String>> simpleResult = new ArrayList<>();
        //假期余额记录
        List<Map<String, Object>> holidayBalancefilterList = new ArrayList<>();
        for (Map<String, Object> map : canUseJqyeList){
            if (DateUtil.getTime(map.get("yqsxrq").toString()).compareTo(DateUtil.getTime(ksrq)) >= 0){
                holidayBalancefilterList.add(map);
            }
        }
        if (holidayBalancefilterList.size() == 0) {
            return;
        }
        //使用的假期类型的额度单位
        String hsdw = Util.null2String(holidayBalancefilterList.get(0).get("hsdw"));
        //20241118需求变更，全天、半天时长根据班次信息获取，以下请假时长分配逻辑的执行前提必须遵守“核算单位为‘天’的假期只能申请全天假、半天假！”
        //所以，只有在全天假和半天假时，leaveDurationMap中才会有请假天数信息；其他请假方式则会请假天数和请假小时数都有
        double totalLeaveDuration;
        double leaveDuration;
        if (hsdw.equals(AccountingUnitEnum.DAY.getKey())) {
            totalLeaveDuration = totalLeaveDays;
            leaveDuration = leaveDays;
        } else {
            totalLeaveDuration = totalLeaveHours;
            leaveDuration = leaveHours;
        }
        boolean jqlxEdited;
        for (Map<String, Object> holidayBalance : holidayBalancefilterList) {
            String jqlxId = holidayBalance.get("jqid").toString();
            Double jqlxEditedUseSc = editedUseJqlxWithEmp.getOrDefault(jqlxId + "_" + empId, (double) 0);
            //使用的的请假项目是否已被编辑
            jqlxEdited = editedLeaveInfo.get(empId + "_" + jqlxId + "_" + ksrq) != null;
            //本次请假日期范围内，已存在该请假类型开始日期相同的请假明细，则不使用该请假类型
            if (jqlxEditedUseSc > 0 || jqlxEdited) {
                continue;
            }
            //1-如果已编辑的请假明细中已有该假期余额id的记录，则满足最小使用时长;如果result的请假明细中已有该假期余额id的记录，则满足最小使用时长
            boolean minLeaveDurationSign = editedUseJqed.get(holidayBalance.get("id").toString()) != null || result.stream().anyMatch(map -> jqlxId.equals(map.get("qjlx")));
            //2-如果编辑记录中不存在，则需要判断是否满足最小使用时长
            if (!minLeaveDurationSign) {
                String minLeaveDuration = Util.null2String(holidayBalance.get("minLeaveDuration"));
                minLeaveDurationSign = "".equals(minLeaveDuration) || totalLeaveDuration >= Double.parseDouble(minLeaveDuration);
            }
            //不满足最小使用时长，直接执行下一条
            if (!minLeaveDurationSign) {
                continue;
            }
            //额定未休时长
            double wxsc = "".equals(Util.null2String(holidayBalance.get("wxsc"))) ? 0 : Double.parseDouble(holidayBalance.get("wxsc").toString());
            if (leaveDuration > 0  && wxsc > 0) {
                detailData.put("qjlx", jqlxId);
                detailData.put("qjlxName", jqlxIdToNameInfo.get(jqlxId));

                if (leaveDuration > wxsc) {
                    detailData.put("qjsc", String.valueOf(wxsc));
                    leaveDuration = Utils.subtract(leaveDuration, wxsc);
                    totalLeaveDuration = Utils.subtract(totalLeaveDuration, wxsc);
                    detailData.put("jqye", holidayBalance.get("id").toString() + "_" + wxsc);
                    wxsc = 0;
                    simpleResult.add(new HashMap<>(detailData));

                } else {
                    detailData.put("qjsc", String.valueOf(leaveDuration));
                    wxsc = Utils.subtract(wxsc, leaveDuration);

                    totalLeaveDuration = Utils.subtract(totalLeaveDuration, leaveDuration);
                    detailData.put("jqye", holidayBalance.get("id").toString() + "_" + leaveDuration);
                    leaveDuration = 0;
                    simpleResult.add(detailData);
                }

                holidayBalance.put("wxsc", wxsc);
            }
        }
        //合并同一假期类型的明细
        if (simpleResult.size() > 0) {
            Map<String, List<Map<String, String>>> simpleResultGroup = simpleResult.stream().collect(Collectors.groupingBy(m -> m.get("qjlx")));
            for (Map.Entry<String, List<Map<String, String>>> entry : simpleResultGroup.entrySet()) {
                double qjscSum = entry.getValue().stream().mapToDouble(f->Double.parseDouble(f.get("qjsc"))).sum();
                String jqyeUseInfo = entry.getValue().stream().map(map -> map.get("jqye")).collect(Collectors.joining(","));
                Map<String, String> resultItem = entry.getValue().get(0);
                resultItem.put("qjsc", String.valueOf(qjscSum));
                resultItem.put("jqye", jqyeUseInfo);
                result.add(resultItem);
            }
        }

        if (hsdw.equals(AccountingUnitEnum.DAY.getKey())) {
            //假别核算单位为‘天’时，需要更新leaveDurationMap中的请假天数和请假小时数
            leaveDurationMap.put("totalLeaveDays", totalLeaveDuration);
            leaveDurationMap.put("leaveDays", leaveDuration);
            double ratedDayHours = dateToRatedHours.getOrDefault(detailData.get("ksrq"), 8.0);
            leaveDurationMap.put("totalLeaveHours", totalLeaveHours - (leaveDays - leaveDuration) * ratedDayHours);
            leaveDurationMap.put("leaveHours", leaveHours - (leaveDays - leaveDuration) * ratedDayHours);
        } else {
            //假别核算单位为‘小时’时，只需要更新leaveDurationMap中的请假小时数
            leaveDurationMap.put("totalLeaveHours", totalLeaveDuration);
            leaveDurationMap.put("leaveHours", leaveDuration);
        }

    }

    /**
     * 收集假期类型关联的可使用的假期余额
     * @param unableUseJqyeIdList 不可使用的假期余额id集合
     * @param editedUseJqed 已编辑过的假期余额id和具体使用的时长映射
     * @param holidayItem 假期类型对应的考勤项目信息
     * @param startDate 开始日期
     * @param leaveEmpId 请假人id
     * @param jbPriorityValue
     */
    private List<Map<String, Object>> collectUsableHolidayBalance(List<String> unableUseJqyeIdList, Map<String, Double> editedUseJqed, Map<String, Object> holidayItem,
                                                                  String startDate, String leaveEmpId, String empName, String modeId, List<String> errorMessage, int jbPriorityValue) {

        String itemId = Util.null2String(holidayItem.get("id"));
        String sql = "select id,jqid,sxrq,ktsc,yxsc,wxsc,yqsxrq,ztsc from uf_jcl_kq_jqye where ygid=? and jqid=? and sxrq<=? and yqsxrq>=? order by yqsxrq, sxrq";
        List<Map<String, Object>> holidayBalanceList = DbTools.getSqlToList(sql, leaveEmpId, itemId, startDate, startDate);
        //获取延期失效日期最晚的一条
        Map<String, Object> maxYqsxrqMap = holidayBalanceList.stream().reduce((m1, m2) -> m2).orElse(null);
        //获取假期额度规则中额度可休次数、单次最小休时长、额度单位
        Map<String, Object> jqedInfo = new HashMap<>();
        String jqedSql = "select id, eddw, dczskxsc, edbxdcxw from uf_jcl_kq_jqed where jb = ? and gzzt = 0";
        List<Map<String, Object>> jqedData = DbTools.getSqlToList(jqedSql, itemId);
        Integer empPriority = null;
        boolean samePriority = false;
        for (Map<String, Object> jqedItem : jqedData) {
            String dataId = jqedItem.get("id").toString();
            //获取假期额度规则适用的人员id列表
//            List<String> empIdList = CommonUtil.getEmpIds(dataId, modeId);
            Map<String, Integer> empInfo = CommonUtil.getEmpIdWithPriority(dataId, modeId);
            if (empInfo.get(leaveEmpId) != null) {
                if (empPriority == null) {
                    jqedInfo = jqedItem;
                    empPriority = empInfo.get(leaveEmpId);
                } else if (empPriority > empInfo.get(leaveEmpId)) {
                    jqedInfo = jqedItem;
                    empPriority = empInfo.get(leaveEmpId);
                } else if (empPriority.equals(empInfo.get(leaveEmpId))) {
                    samePriority = true;
                }

            }
        }
        if (empPriority != null && samePriority) {
            String message = empName;
            message = message + "可使用的假别—‘" + holidayItem.get("mc") + "’同一优先级情况下存在超过一种适用的假期额度规则，无法确定唯一规则！";
            errorMessage.add(message);
            return new ArrayList<>();
        }
        //假期的核算单位，0-天、1-小时
        String hsdw = Util.null2String(holidayItem.get("hsdw"));
        //假期额度的额度单位，0-天、1-小时
        String eddw = Util.null2String(jqedInfo.get("eddw"));
        //单次最小休时长，20241118需求变更，时长不进行倍率转换，按照原本核算单位进行收集
        String minDuration = Util.null2String(jqedInfo.get("dczskxsc"));
        Double minLeaveDuration = !"".equals(minDuration) ? Double.parseDouble(minDuration) : null;
        //额度可修次数
        String allowLeaveNumStr = Util.null2String(jqedInfo.get("edbxdcxw"));
        if (!"".equals(allowLeaveNumStr) && holidayBalanceList.size() > 0) {
            //查询请假记录中，人员id+假期类别id情况下的结果，遍历每条主表数据下的明细数据中使用了哪些假期余额id
            sql = "select a.id,dt2.jqye from uf_jcl_kq_qjjl a left join uf_jcl_kq_qjjl_dt1 dt1 on dt1.mainid = a.id left join uf_jcl_kq_qjjl_dt2 dt2 on dt2.glmxid = dt1.glmxid " +
                    "where a.jqlx = " + itemId + " and qjry = " + leaveEmpId + " and a.cxqj = 0 and dt1.cxqj = 0 and a.jlzt in (0, 1) and dt1.glmxid is not null";
            List<Map<String, Object>> leaveList = DbTools.getSqlToList(sql);
            Map<String, Integer> jqyeUseNumInfo = new HashMap<>();
            List<String> leaveUseList = new ArrayList<>();
            Integer useNum;
            for (Map<String, Object> leaveItem : leaveList) {
                String jqyeId = Util.null2String(leaveItem.get("jqye"));
                if (!"".equals(jqyeId)) {
                    String leaveInfo = leaveItem.get("id").toString() + "_" + jqyeId;
                    if (leaveUseList.size() == 0) {
                        leaveUseList.add(leaveInfo);
                        jqyeUseNumInfo.put(jqyeId, 1);
                    } else if (!leaveUseList.contains(leaveInfo)) {
                        leaveUseList.add(leaveInfo);
                        useNum = jqyeUseNumInfo.get(jqyeId);
                        jqyeUseNumInfo.put(jqyeId, useNum == null ? 1 : ++useNum);
                    }
                }
            }
            //筛选次数未使用完的假期余额
            holidayBalanceList = holidayBalanceList.stream().filter(f -> {
                int beforeUseNum = jqyeUseNumInfo.get(f.get("id").toString()) == null ? 0 : jqyeUseNumInfo.get(f.get("id").toString());
                return Integer.parseInt(allowLeaveNumStr) > beforeUseNum;
            }).collect(Collectors.toList());
        }
        //判断是否可预支
        String allowAdvance = Util.null2String(holidayItem.get("yxyz"));
        String advanceSc = Util.null2String(holidayItem.get("zdyzsl"));
        //遍历可使用的假期余额，去除已编辑请假明细记录中使用的时长
        List<Map<String, Object>> canUseHolidayBalanceList = new ArrayList<>();
        for (Map<String, Object> holidayBalance : holidayBalanceList) {
            String jqyeId = Util.null2String(holidayBalance.get("id"));
            if (!unableUseJqyeIdList.contains(jqyeId)) {
                //20241118需求变更，时长不进行核算单位倍率转换，按照原本核算单位进行收集
                double wxsc = Utils.convertDouble(holidayBalance.get("wxsc"));
                //如果该条假期余额可以预支，则在原有未休时长基础上增加预支时长
                if ("1".equals(allowAdvance) && maxYqsxrqMap != null && Util.null2String(maxYqsxrqMap.get("id")).equals(jqyeId)) {
                    wxsc = wxsc + Utils.convertDouble(advanceSc);
                }
                double editedUseSc = Utils.convertDouble(editedUseJqed.get(jqyeId));
                double realWxsc = wxsc - editedUseSc;
                if ((minLeaveDuration == null || wxsc >= minLeaveDuration) && realWxsc > 0) {
                    holidayBalance.put("wxsc", realWxsc);
                    holidayBalance.put("minLeaveDuration", minLeaveDuration);
                    holidayBalance.put("eddw", eddw);
                    holidayBalance.put("hsdw", hsdw);
                    holidayBalance.put("jbPriorityValue", jbPriorityValue);
                    canUseHolidayBalanceList.add(holidayBalance);
                }
            }
        }
        return canUseHolidayBalanceList;
    }

    /**
     * 请假人的请假区间内的排班结果
     * @param leaveEmpIdList 请假人id集合
     * @param startDate 开始日期
     * @param endDate 结束日期
     */
    private Map<String, List<Map<String, Object>>> getScheduleInfoWithEmpId(List<String> leaveEmpIdList, String startDate, String endDate) {
        Map<String, List<Map<String, Object>>> scheduleInfo = new HashMap<>();
        Map<String,Object> params = new HashMap<>();
        params.put("tableName", "uf_pbjg");
        params.put("startDate", startDate);
        params.put("endDate", endDate);
        params.put("pblx", "0");
        params.put("current", "1");
        params.put("pageSize", "999");
        params.put("recurrence", "1");
        ExtensionClassHolder.setPersonBelongGroupThreadLocal(PersongroupCommonUtil.getAllpersonBelongGroup());
        for (String empId : leaveEmpIdList) {
            params.put("pbdx", empId);
            Map<String,Object> schedulingResultsMap = schedulingResultsService.queryDataTableActualUse(params);
            scheduleInfo.put(empId, (List<Map<String, Object>>) schedulingResultsMap.get("data"));
        }
        return scheduleInfo;
    }

    /**
     * 请假人的请假区间内属于休息日的日期集合
     * @param leaveEmpIdList 请假人id集合
     * @param leaveType 假期类型id，即考勤项目id
     * @param startDate 开始日期
     * @param endDate 结束日期
     */
    private Map<String, List<String>> getRestDayWithEmpId(List<String> leaveEmpIdList, String leaveType, String startDate, String endDate) {
        Map<String, List<String>> restDayInfoWithEmpId = new HashMap<>();

        Map<String, Object> restDayParam = new HashMap<>();
        restDayParam.put("itemId", leaveType);
        restDayParam.put("startDate", startDate);
        restDayParam.put("endDate", endDate);

        List<String> dateTypeList = new ArrayList<>();
        dateTypeList.add(DateTypeEnum.HOLIDAY.getKey());
        dateTypeList.add(DateTypeEnum.PUBLIC_RESTDAY.getKey());
        dateTypeList.add(DateTypeEnum.EXCHANGE_LEAVEDAY.getKey());
        Map<String, Object> restDayInfo;
        List<Map<String,Object>> dataList;
        List<String> restDateList = new ArrayList<>();
        for (String empId : leaveEmpIdList) {
            restDayParam.put("userId", empId);
            restDayInfo = getRestDayInterval(restDayParam);
            dataList = (List<Map<String, Object>>) restDayInfo.get("data");
            if (dataList != null && dataList.size() > 0) {
                restDateList = dataList.stream()
                        .filter(f -> dateTypeList.contains(Util.null2String(f.get("rqlx"))))
                        .map(e -> Util.null2String(e.get("rq")))
                        .collect(Collectors.toList());
            }
            restDayInfoWithEmpId.put(empId, restDateList);
        }
        return restDayInfoWithEmpId;
    }
}
