package com.engine.jucailinkq.common.cmd;

import com.engine.common.biz.AbstractCommonCommand;
import com.engine.common.entity.BizLogContext;
import com.engine.core.interceptor.CommandContext;
import com.engine.jucailinkq.common.util.DbTools;
import com.engine.jucailinkq.common.util.ExtensionClassHolder;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.google.common.collect.Sets;
import weaver.general.Util;
import weaver.hrm.company.DepartmentComInfo;

import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

public class GetAllPersonBySuitOrganzationCmd extends AbstractCommonCommand<Map<String, Object>> {
    public GetAllPersonBySuitOrganzationCmd(Map<String, Object> params) {
        this.params = params;
    }

    @Override
    public BizLogContext getLogContext() {
        return null;
    }

    @Override
    public Map<String, Object> execute(CommandContext commandContext) {
        String modeId = Util.null2String(params.get("modeId"));
        String sql = "select dxlx,dataid,dx,aqjb from uf_jcl_syzz where modeid=?";
        List<Map<String, Object>> organizationList = DbTools.getSqlToList(sql, modeId);
        Map<String,Object> resultMap = Maps.newHashMap();

        Map<String, List<Map<String, Object>>> organizationListGroupBydxlx = organizationList.stream().collect(Collectors.groupingBy(e -> e.get("dxlx").toString()));
        //对象类型为人员
        List<Map<String, Object>> personOrganizationList = organizationListGroupBydxlx.get("0");
        //对象类型为人员组织
        List<Map<String, Object>> personGroupOrganizationList = organizationListGroupBydxlx.get("1");
        //对象类型为部门
        List<Map<String, Object>> departmentOrganizationList = organizationListGroupBydxlx.get("2");
        //对象类型为分部
        List<Map<String, Object>> subCompanyOrganizationList = organizationListGroupBydxlx.get("3");

        //人力资源id，
        Map<String, List<Map<String, String>>> personData = Maps.newHashMap();
        //人员分组id，
        Map<String, List<Map<String, String>>> personOrganzationMap = Maps.newHashMap();
        //部门id，
        Map<String, List<Map<String, String>>> departMentMap = Maps.newHashMap();
        //分部id
        Map<String, List<Map<String, String>>> subCompanyMap = Maps.newHashMap();


        if (personOrganizationList != null) {
            for (Map<String, Object> personOrganization : personOrganizationList) {
                String dx = Util.null2String(personOrganization.get("dx"));
                String userId = dx.split("-")[0];
                String dataId = Util.null2String(personOrganization.get("dataid"));

                List<Map<String, String>> dataList2 = personData.get(userId);
                if (dataList2 == null) {
                    dataList2 = Lists.newArrayList();
                    personData.put(userId, dataList2);
                }
                if (dataList2.stream().filter(e -> e.get("dataid").equals(dataId)).collect(Collectors.toList()).size() == 0) {
                    Map<String, String> dataMap = Maps.newHashMap();
                    dataMap.put("dataid", dataId);
                    dataMap.put("priority", "0");
                    dataMap.put("seclevel",Util.null2String(personOrganization.get("aqjb")));
                    dataList2.add(dataMap);
                }
            }
        }
        if (personGroupOrganizationList != null){
            for (Map<String, Object> personGroupOrganization : personGroupOrganizationList) {
                String dx = Util.null2String(personGroupOrganization.get("dx"));
                String personGroupId = dx.split("-")[0];
                String dataId = Util.null2String(personGroupOrganization.get("dataid"));
                List<Map<String, String>> dataList = personOrganzationMap.get(personGroupId);
                if (dataList == null) {
                    dataList = Lists.newArrayList();
                    personOrganzationMap.put(personGroupId, dataList);
                }
                if (dataList.stream().filter(e -> e.get("dataid").equals(dataId)).collect(Collectors.toList()).size() == 0) {
                    Map<String, String> dataMap = Maps.newHashMap();
                    dataMap.put("dataid", dataId);
                    dataMap.put("priority", "0");
                    dataMap.put("seclevel",Util.null2String(personGroupOrganization.get("aqjb")));

                    dataList.add(dataMap);
                }
            }
        }

        if (departmentOrganizationList != null) {
            for (Map<String, Object> departmentOrganization : departmentOrganizationList) {
                String dx = Util.null2String(departmentOrganization.get("dx"));
                String departmentId = dx.split("-")[0];
                String dataId = Util.null2String(departmentOrganization.get("dataid"));
                List<Map<String, String>> dataList = departMentMap.get(departmentId);
                if (dataList == null) {
                    dataList = Lists.newArrayList();
                    departMentMap.put(departmentId, dataList);
                }
                if (dataList.stream().filter(e -> e.get("dataid").equals(dataId)).collect(Collectors.toList()).size() == 0) {
                    Map<String, String> dataMap = Maps.newHashMap();
                    dataMap.put("dataid", dataId);
                    dataMap.put("priority", "2");
                    dataMap.put("seclevel",Util.null2String(departmentOrganization.get("aqjb")));
                    dataList.add(dataMap);
                }
            }
        }
        if (subCompanyOrganizationList != null) {
            for (Map<String, Object> subCompanyOrganization : subCompanyOrganizationList) {
                String dx = Util.null2String(subCompanyOrganization.get("dx"));
                String subCompanyId = dx.split("-")[0];
                String dataId = Util.null2String(subCompanyOrganization.get("dataid"));
                List<Map<String, String>> dataList = subCompanyMap.get(subCompanyId);
                if (dataList == null) {
                    dataList = Lists.newArrayList();
                    subCompanyMap.put(subCompanyId, dataList);
                }
                if (dataList.stream().filter(e -> e.get("dataid").equals(dataId)).collect(Collectors.toList()).size() == 0) {
                    Map<String, String> dataMap = Maps.newHashMap();
                    dataMap.put("dataid", dataId);
                    dataMap.put("priority", "3");
                    dataMap.put("seclevel",Util.null2String(subCompanyOrganization.get("aqjb")));
                    dataList.add(dataMap);
                }
            }
        }
        resultMap.put("personData",personData);
        resultMap.put("personOrganzationMap",personOrganzationMap);
        resultMap.put("departMentMap",departMentMap);
        resultMap.put("subCompanyMap",subCompanyMap);

        return resultMap;
    }
}
